@@ -1,4 +1,4 @@
-// Copyright (c) 2017-2021, The rav1e contributors. All rights reserved
+// Copyright (c) 2017-2022, The rav1e contributors. All rights reserved
 //
 // This source code is subject to the terms of the BSD 2 Clause License and
 // the Alliance for Open Media Patent License 1.0. If the BSD 2 Clause License
@@ -16,16 +16,27 @@
 #![allow(clippy::verbose_bit_mask)]
 #![allow(clippy::unreadable_literal)]
 #![allow(clippy::many_single_char_names)]
-#![warn(clippy::expl_impl_clone_on_copy)]
+// Performance lints
 #![warn(clippy::linkedlist)]
-#![warn(clippy::map_flatten)]
-#![warn(clippy::mem_forget)]
-#![warn(clippy::mut_mut)]
+#![warn(clippy::missing_const_for_fn)]
 #![warn(clippy::mutex_integer)]
+#![warn(clippy::suboptimal_flops)]
+// Correctness lints
+#![warn(clippy::expl_impl_clone_on_copy)]
+#![warn(clippy::mem_forget)]
+#![warn(clippy::path_buf_push_overwrite)]
+// Clarity/formatting lints
+#![warn(clippy::map_flatten)]
+#![warn(clippy::mut_mut)]
 #![warn(clippy::needless_borrow)]
 #![warn(clippy::needless_continue)]
-#![warn(clippy::path_buf_push_overwrite)]
 #![warn(clippy::range_plus_one)]
+// Documentation lints
+#![warn(clippy::doc_markdown)]
+#![warn(clippy::missing_errors_doc)]
+#![warn(clippy::missing_panics_doc)]
+// FIXME: Temporarily disabled due to https://github.com/rust-lang/rust-clippy/issues/9142
+#![allow(clippy::undocumented_unsafe_blocks)]
 
 #[macro_use]
 extern crate log;
@@ -115,7 +126,7 @@ impl<D: Decoder> Source<D> {
 }
 
 fn do_encode<T: Pixel, D: Decoder>(
-  cfg: Config, verbose: Verbose, mut progress: ProgressInfo,
+  cfg: Config, verbose: Verboseness, mut progress: ProgressInfo,
   output: &mut dyn Muxer, mut source: Source<D>, pass1file: Option<File>,
   pass2file: Option<File>,
   mut y4m_enc: Option<y4m::Encoder<Box<dyn Write + Send>>>,
@@ -267,9 +278,9 @@ fn do_encode<T: Pixel, D: Decoder>(
           metrics_enabled,
         );
 
-        if verbose != Verbose::Quiet {
+        if verbose != Verboseness::Quiet {
           progress.add_frame(summary.clone());
-          if verbose == Verbose::Verbose {
+          if verbose == Verboseness::Verbose {
             info!("{} - {}", summary, progress);
           } else {
             // Print a one-line progress indicator that overrides itself with every update
@@ -278,12 +289,12 @@ fn do_encode<T: Pixel, D: Decoder>(
         }
       }
 
-      if verbose != Verbose::Quiet {
-        if verbose == Verbose::Verbose {
+      if verbose != Verboseness::Quiet {
+        if verbose == Verboseness::Verbose {
           // Clear out the temporary progress indicator
           eprint!("\r");
         }
-        progress.print_summary(verbose == Verbose::Verbose);
+        progress.print_summary(verbose == Verboseness::Verbose);
       }
 
       // receive_packet.result()
@@ -335,12 +346,10 @@ fn init_logger() {
     }
   }
 
-  // this can be changed to flatten
   let level = std::env::var("RAV1E_LOG")
     .ok()
-    .map(|l| log::LevelFilter::from_str(&l).ok())
-    .unwrap_or(Some(log::LevelFilter::Info))
-    .unwrap();
+    .and_then(|l| log::LevelFilter::from_str(&l).ok())
+    .unwrap_or(log::LevelFilter::Info);
 
   fern::Dispatch::new()
     .format(move |out, message, record| {
@@ -372,6 +381,7 @@ cfg_if::cfg_if! {
     }
   } else {
     fn print_rusage() {
+      // SAFETY: This uses an FFI, it is safe because we call it correctly.
       let (utime, stime, maxrss) = unsafe {
         let mut usage = std::mem::zeroed();
         let _ = libc::getrusage(libc::RUSAGE_SELF, &mut usage);
@@ -402,8 +412,21 @@ fn run() -> Result<(), error::CliError> {
       .saturating_add(1024),
   };
   let mut y4m_dec = match y4m::Decoder::new_with_limits(cli.io.input, limit) {
-    Err(_) => {
-      return Err(CliError::new("Could not input video. Is it a y4m file?"))
+    Err(e) => {
+      return Err(CliError::new(match e {
+        y4m::Error::ParseError(_) => {
+          "Could not parse input video. Is it a y4m file?"
+        }
+        y4m::Error::IoError(_) => {
+          "Could not read input file. Check that the path is correct and you have read permissions."
+        }
+        y4m::Error::UnknownColorspace => {
+          "Unknown colorspace or unsupported bit depth."
+        }
+        y4m::Error::OutOfMemory => "The video's frame size exceeds the limit.",
+        y4m::Error::EOF => "Unexpected end of input.",
+        y4m::Error::BadInput => "Bad y4m input parameters provided.",
+      }))
     }
     Ok(d) => d,
   };
@@ -448,6 +471,25 @@ fn run() -> Result<(), error::CliError> {
     cli.enc.time_base = video_info.time_base;
   }
 
+  if cli.photon_noise > 0 && cli.enc.film_grain_params.is_none() {
+    cli.enc.film_grain_params = Some(vec![generate_photon_noise_params(
+      0,
+      u64::MAX,
+      NoiseGenArgs {
+        iso_setting: cli.photon_noise as u32 * 100,
+        width: video_info.width as u32,
+        height: video_info.height as u32,
+        transfer_function: if cli.enc.is_hdr() {
+          TransferFunction::SMPTE2084
+        } else {
+          TransferFunction::BT1886
+        },
+        chroma_grain: false,
+        random_seed: None,
+      },
+    )]);
+  }
+
   let mut rc = RateControlConfig::new();
 
   let pass2file = match cli.pass2file_name {
@@ -484,7 +526,7 @@ fn run() -> Result<(), error::CliError> {
   };
 
   let cfg = Config::new()
-    .with_encoder_config(cli.enc)
+    .with_encoder_config(cli.enc.clone())
     .with_threads(cli.threads)
     .with_rate_control(rc)
     .with_parallel_gops(cli.slots);
@@ -510,7 +552,7 @@ fn run() -> Result<(), error::CliError> {
 
   let tiling =
     cfg.tiling_info().map_err(|e| e.context("Invalid configuration"))?;
-  if cli.verbose != Verbose::Quiet {
+  if cli.verbose != Verboseness::Quiet {
     info!("CPU Feature Level: {}", CpuFeatureLevel::default());
 
     info!(
