@@ -1,4 +1,4 @@
-// Copyright (c) 2017-2021, The rav1e contributors. All rights reserved
+// Copyright (c) 2017-2022, The rav1e contributors. All rights reserved
 //
 // This source code is subject to the terms of the BSD 2 Clause License and
 // the Alliance for Open Media Patent License 1.0. If the BSD 2 Clause License
@@ -16,16 +16,27 @@
 #![allow(clippy::verbose_bit_mask)]
 #![allow(clippy::unreadable_literal)]
 #![allow(clippy::many_single_char_names)]
-#![warn(clippy::expl_impl_clone_on_copy)]
+// Performance lints
 #![warn(clippy::linkedlist)]
-#![warn(clippy::map_flatten)]
-#![warn(clippy::mem_forget)]
-#![warn(clippy::mut_mut)]
+#![warn(clippy::missing_const_for_fn)]
 #![warn(clippy::mutex_integer)]
+#![warn(clippy::suboptimal_flops)]
+// Correctness lints
+#![warn(clippy::expl_impl_clone_on_copy)]
+#![warn(clippy::mem_forget)]
+#![warn(clippy::path_buf_push_overwrite)]
+// Clarity/formatting lints
+#![warn(clippy::map_flatten)]
+#![warn(clippy::mut_mut)]
 #![warn(clippy::needless_borrow)]
 #![warn(clippy::needless_continue)]
-#![warn(clippy::path_buf_push_overwrite)]
 #![warn(clippy::range_plus_one)]
+// Documentation lints
+#![warn(clippy::doc_markdown)]
+#![warn(clippy::missing_errors_doc)]
+#![warn(clippy::missing_panics_doc)]
+// FIXME: Temporarily disabled due to https://github.com/rust-lang/rust-clippy/issues/9142
+#![allow(clippy::undocumented_unsafe_blocks)]
 
 #[macro_use]
 extern crate log;
@@ -43,6 +54,7 @@ use crate::error::*;
 use crate::stats::*;
 use rav1e::config::CpuFeatureLevel;
 use rav1e::prelude::*;
+use rust_hawktracer::*;
 
 use crate::decoder::{Decoder, FrameBuilder, VideoDetails};
 use crate::muxer::*;
@@ -86,12 +98,14 @@ impl<D: Decoder> Source<D> {
         Self { limit, input, count: 0, exit_requested, }
       }
     } else {
+      #[allow(clippy::missing_const_for_fn)]
       fn new(limit: usize, input: D) -> Self {
         Self { limit, input, count: 0, }
       }
     }
   }
 
+  #[hawktracer(Source_read_frame)]
   fn read_frame<T: Pixel>(
     &mut self, ctx: &mut Context<T>, video_info: VideoDetails,
   ) -> Result<(), CliError> {
@@ -127,6 +141,7 @@ impl<D: Decoder> Source<D> {
 
 // Encode and write a frame.
 // Returns frame information in a `Result`.
+#[hawktracer(process_frame)]
 fn process_frame<T: Pixel, D: Decoder>(
   ctx: &mut Context<T>, output_file: &mut dyn Muxer, source: &mut Source<D>,
   pass1file: Option<&mut File>, pass2file: Option<&mut File>,
@@ -161,7 +176,7 @@ fn process_frame<T: Pixel, D: Decoder>(
   let (ret, emit_pass_data) = match pkt_wrapped {
     Ok(pkt) => {
       output_file.write_frame(
-        pkt.input_frameno as u64,
+        pkt.input_frameno,
         pkt.data.as_ref(),
         pkt.frame_type,
       );
@@ -176,28 +191,24 @@ fn process_frame<T: Pixel, D: Decoder>(
         y4m_details.chroma_sampling,
         metrics_cli,
       ));
-      (Ok(Some(frame_summaries)), true)
+      Ok((Some(frame_summaries), true))
     }
     Err(EncoderStatus::NeedMoreData) => {
       source.read_frame(ctx, y4m_details)?;
-      (Ok(Some(frame_summaries)), false)
+      Ok((Some(frame_summaries), false))
     }
     Err(EncoderStatus::EnoughData) => {
-      unreachable!();
+      unreachable!()
     }
-    Err(EncoderStatus::LimitReached) => (Ok(None), true),
+    Err(EncoderStatus::LimitReached) => Ok((None, true)),
     Err(e @ EncoderStatus::Failure) => {
-      (Err(e.context("Failed to encode video")), false)
+      Err(e.context("Failed to encode video"))
     }
     Err(e @ EncoderStatus::NotReady) => {
-      (Err(e.context("Mismanaged handling of two-pass stats data")), false)
+      Err(e.context("Mismanaged handling of two-pass stats data"))
     }
-    Err(EncoderStatus::Encoded) => (Ok(Some(frame_summaries)), true),
-  };
-
-  if ret.is_err() {
-    return ret;
-  }
+    Err(EncoderStatus::Encoded) => Ok((Some(frame_summaries), true)),
+  }?;
 
   // Save first pass data from pass 1.
   if let Some(passfile) = pass1file.as_mut() {
@@ -234,11 +245,11 @@ fn process_frame<T: Pixel, D: Decoder>(
     }
   }
 
-  ret
+  Ok(ret)
 }
 
 fn do_encode<T: Pixel, D: Decoder>(
-  cfg: Config, verbose: Verbose, mut progress: ProgressInfo,
+  cfg: Config, verbose: Verboseness, mut progress: ProgressInfo,
   output: &mut dyn Muxer, mut source: Source<D>, mut pass1file: Option<File>,
   mut pass2file: Option<File>,
   mut y4m_enc: Option<y4m::Encoder<Box<dyn Write + Send>>>,
@@ -270,10 +281,10 @@ fn do_encode<T: Pixel, D: Decoder>(
     y4m_enc.as_mut(),
     metrics_enabled,
   )? {
-    if verbose != Verbose::Quiet {
+    if verbose != Verboseness::Quiet {
       for frame in frame_info {
         progress.add_frame(frame.clone());
-        if verbose == Verbose::Verbose {
+        if verbose == Verboseness::Verbose {
           info!("{} - {}", frame, progress);
         } else {
           // Print a one-line progress indicator that overrides itself with every update
@@ -284,19 +295,18 @@ fn do_encode<T: Pixel, D: Decoder>(
       output.flush().unwrap();
     }
   }
-  if verbose != Verbose::Quiet {
-    if verbose == Verbose::Verbose {
+  if verbose != Verboseness::Quiet {
+    if verbose == Verboseness::Verbose {
       // Clear out the temporary progress indicator
       eprint!("\r");
     }
-    progress.print_summary(verbose == Verbose::Verbose);
+    progress.print_summary(verbose == Verboseness::Verbose);
   }
   Ok(())
 }
 
 fn main() {
   #[cfg(feature = "tracing")]
-  use rust_hawktracer::*;
   init_logger();
 
   #[cfg(feature = "tracing")]
@@ -313,6 +323,7 @@ fn main() {
   });
 }
 
+#[cfg(feature = "tracing")]
 fn init_logger() {
   use std::str::FromStr;
   fn level_colored(l: log::Level) -> console::StyledObject<&'static str> {
@@ -327,12 +338,10 @@ fn init_logger() {
     }
   }
 
-  // this can be changed to flatten
   let level = std::env::var("RAV1E_LOG")
     .ok()
-    .map(|l| log::LevelFilter::from_str(&l).ok())
-    .unwrap_or(Some(log::LevelFilter::Info))
-    .unwrap();
+    .and_then(|l| log::LevelFilter::from_str(&l).ok())
+    .unwrap_or(log::LevelFilter::Info);
 
   fern::Dispatch::new()
     .format(move |out, message, record| {
@@ -363,6 +372,7 @@ cfg_if::cfg_if! {
     }
   } else {
     fn print_rusage() {
+      // SAFETY: This uses an FFI, it is safe because we call it correctly.
       let (utime, stime, maxrss) = unsafe {
         let mut usage = std::mem::zeroed();
         let _ = libc::getrusage(libc::RUSAGE_SELF, &mut usage);
@@ -396,13 +406,17 @@ fn run() -> Result<(), error::CliError> {
     Err(e) => {
       return Err(CliError::new(match e {
         y4m::Error::ParseError(_) => {
-          "Could not input video. Is it a y4m file?"
+          "Could not parse input video. Is it a y4m file?"
+        }
+        y4m::Error::IoError(_) => {
+          "Could not read input file. Check that the path is correct and you have read permissions."
         }
         y4m::Error::UnknownColorspace => {
           "Unknown colorspace or unsupported bit depth."
         }
         y4m::Error::OutOfMemory => "The video's frame size exceeds the limit.",
-        _ => unreachable!(),
+        y4m::Error::EOF => "Unexpected end of input.",
+        y4m::Error::BadInput => "Bad y4m input parameters provided.",
       }))
     }
     Ok(d) => d,
@@ -443,6 +457,25 @@ fn run() -> Result<(), error::CliError> {
     cli.enc.time_base = video_info.time_base;
   }
 
+  if cli.photon_noise > 0 && cli.enc.film_grain_params.is_none() {
+    cli.enc.film_grain_params = Some(vec![generate_photon_noise_params(
+      0,
+      u64::MAX,
+      NoiseGenArgs {
+        iso_setting: cli.photon_noise as u32 * 100,
+        width: video_info.width as u32,
+        height: video_info.height as u32,
+        transfer_function: if cli.enc.is_hdr() {
+          TransferFunction::SMPTE2084
+        } else {
+          TransferFunction::BT1886
+        },
+        chroma_grain: false,
+        random_seed: None,
+      },
+    )]);
+  }
+
   let mut rc = RateControlConfig::new();
 
   let pass2file = match cli.pass2file_name {
@@ -479,7 +512,7 @@ fn run() -> Result<(), error::CliError> {
   };
 
   let cfg = Config::new()
-    .with_encoder_config(cli.enc)
+    .with_encoder_config(cli.enc.clone())
     .with_threads(cli.threads)
     .with_rate_control(rc);
 
@@ -504,7 +537,7 @@ fn run() -> Result<(), error::CliError> {
 
   let tiling =
     cfg.tiling_info().map_err(|e| e.context("Invalid configuration"))?;
-  if cli.verbose != Verbose::Quiet {
+  if cli.verbose != Verboseness::Quiet {
     info!("CPU Feature Level: {}", CpuFeatureLevel::default());
 
     info!(
