@@ -154,7 +154,7 @@ impl ProgressInfo {
 
   // Elapsed time in seconds
   pub fn elapsed_time(&self) -> u64 {
-    Instant::now().duration_since(self.time_started).as_secs() as u64
+    Instant::now().duration_since(self.time_started).as_secs()
   }
 
   // Number of frames of given type which appear in the video
@@ -676,20 +676,20 @@ impl fmt::Display for ProgressInfo {
   fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
     if let Some(total_frames) = self.total_frames {
       write!(
-                f,
-                "encoded {}/{} frames, {:.3} fps, {:.2} Kb/s, est. size: {:.2} MB, est. time: {},  elap. time: {}",
-                self.frames_encoded(),
-                total_frames,
-                self.encoding_fps(),
-                self.bitrate() as f64 / 1000f64,
-                self.estimated_size() as f64 / (1024 * 1024) as f64,
-                secs_to_human_time(self.estimated_time()),
-                secs_to_human_time(self.elapsed_time())
-            )
+        f,
+        "encoded {}/{} frames, {:.3} fps, {:.2} Kb/s, est: {}, {:.2} MB, elapsed: {}",
+        self.frames_encoded(),
+        total_frames,
+        self.encoding_fps(),
+        self.bitrate() as f64 / 1000f64,
+        secs_to_human_time(self.estimated_time()),
+        self.estimated_size() as f64 / (1024 * 1024) as f64,
+        secs_to_human_time(self.elapsed_time())
+      )
     } else {
       write!(
         f,
-        "encoded {} frames, {:.3} fps, {:.2} Kb/s, elap. time: {}",
+        "encoded {} frames, {:.3} fps, {:.2} Kb/s, elapsed: {}",
         self.frames_encoded(),
         self.encoding_fps(),
         self.bitrate() as f64 / 1000f64,
@@ -732,7 +732,7 @@ pub struct QualityMetrics {
   pub vmaf: Option<f64>,
 }
 
-#[derive(Debug, Clone, Copy, PartialEq)]
+#[derive(Debug, Clone, Copy, PartialEq, Eq)]
 pub enum MetricsEnabled {
   /// Don't calculate any metrics.
   None,
@@ -742,47 +742,35 @@ pub enum MetricsEnabled {
   All,
 }
 
-#[hawktracer(calculate_frame_metrics)]
 pub fn calculate_frame_metrics<T: Pixel>(
   frame1: &Frame<T>, frame2: &Frame<T>, bit_depth: usize, cs: ChromaSampling,
   metrics: MetricsEnabled,
 ) -> QualityMetrics {
-  let frame1_info = FrameInfo {
-    planes: frame1.planes.clone(),
-    bit_depth,
-    chroma_sampling: cs,
-  };
-
-  let frame2_info = FrameInfo {
-    planes: frame2.planes.clone(),
-    bit_depth,
-    chroma_sampling: cs,
-  };
-
   match metrics {
     MetricsEnabled::None => QualityMetrics::default(),
     MetricsEnabled::Psnr => QualityMetrics {
       psnr: Some(
-        psnr::calculate_frame_psnr(&frame1_info, &frame2_info).unwrap(),
+        psnr::calculate_frame_psnr(frame1, frame2, bit_depth, cs).unwrap(),
       ),
       ..Default::default()
     },
     MetricsEnabled::All => {
       let mut metrics = QualityMetrics {
         psnr: Some(
-          psnr::calculate_frame_psnr(&frame1_info, &frame2_info).unwrap(),
+          psnr::calculate_frame_psnr(frame1, frame2, bit_depth, cs).unwrap(),
         ),
         psnr_hvs: Some(
-          psnr_hvs::calculate_frame_psnr_hvs(&frame1_info, &frame2_info)
+          psnr_hvs::calculate_frame_psnr_hvs(frame1, frame2, bit_depth, cs)
             .unwrap(),
         ),
         ..Default::default()
       };
-      let ssim = ssim::calculate_frame_ssim(&frame1_info, &frame2_info);
+      let ssim = ssim::calculate_frame_ssim(frame1, frame2, bit_depth, cs);
       metrics.ssim = Some(ssim.unwrap());
-      let ms_ssim = ssim::calculate_frame_msssim(&frame1_info, &frame2_info);
+      let ms_ssim =
+        ssim::calculate_frame_msssim(frame1, frame2, bit_depth, cs);
       metrics.ms_ssim = Some(ms_ssim.unwrap());
-      let ciede = ciede::calculate_frame_ciede(&frame1_info, &frame2_info);
+      let ciede = ciede::calculate_frame_ciede(frame1, frame2, bit_depth, cs);
       metrics.ciede = Some(ciede.unwrap());
       // TODO APSNR
       // TODO VMAF
