@@ -10,6 +10,7 @@
 use super::*;
 
 use crate::predict::PredictionMode;
+use rust_hawktracer::*;
 
 pub const MAX_PLANES: usize = 3;
 
@@ -34,7 +35,7 @@ pub const COMP_GROUP_IDX_CONTEXTS: usize = 6;
 pub const COEFF_CONTEXT_MAX_WIDTH: usize = MAX_TILE_WIDTH / MI_SIZE;
 
 /// Absolute offset in blocks, where a block is defined
-/// to be an N*N square where N = (1 << BLOCK_TO_PLANE_SHIFT).
+/// to be an `N*N` square where `N == (1 << BLOCK_TO_PLANE_SHIFT)`.
 #[derive(Clone, Copy, Debug, Default, PartialEq, Eq)]
 pub struct BlockOffset {
   pub x: usize,
@@ -42,12 +43,12 @@ pub struct BlockOffset {
 }
 
 /// Absolute offset in blocks inside a plane, where a block is defined
-/// to be an N*N square where N = (1 << BLOCK_TO_PLANE_SHIFT).
+/// to be an `N*N` square where `N == (1 << BLOCK_TO_PLANE_SHIFT)`.
 #[derive(Clone, Copy, Debug, PartialEq, Eq)]
 pub struct PlaneBlockOffset(pub BlockOffset);
 
 /// Absolute offset in blocks inside a tile, where a block is defined
-/// to be an N*N square where N = (1 << BLOCK_TO_PLANE_SHIFT).
+/// to be an `N*N` square where `N == (1 << BLOCK_TO_PLANE_SHIFT)`.
 #[derive(Clone, Copy, Debug, Default, PartialEq, Eq)]
 pub struct TileBlockOffset(pub BlockOffset);
 
@@ -370,9 +371,9 @@ impl<'a> BlockContext<'a> {
       (tx_size.width() as u8, tx_size.height() as u8)
     };
 
-    let above_ctx = &mut self.above_tx_context[bo.0.x..bo.0.x + n4_w as usize];
+    let above_ctx = &mut self.above_tx_context[bo.0.x..bo.0.x + n4_w];
     let left_ctx =
-      &mut self.left_tx_context[bo.y_in_sb()..bo.y_in_sb() + n4_h as usize];
+      &mut self.left_tx_context[bo.y_in_sb()..bo.y_in_sb() + n4_h];
 
     for v in above_ctx[0..n4_w].iter_mut() {
       *v = tx_w;
@@ -712,7 +713,7 @@ impl<'a> ContextWriter<'a> {
     };
     let above_ctx = intra_mode_context[above_mode as usize];
     let left_ctx = intra_mode_context[left_mode as usize];
-    let cdf = &mut self.fc.kf_y_cdf[above_ctx][left_ctx];
+    let cdf = &self.fc.kf_y_cdf[above_ctx][left_ctx];
     symbol_with_update!(self, w, mode as u32, cdf);
   }
 
@@ -724,8 +725,7 @@ impl<'a> ContextWriter<'a> {
   pub fn write_intra_mode<W: Writer>(
     &mut self, w: &mut W, bsize: BlockSize, mode: PredictionMode,
   ) {
-    let cdf =
-      &mut self.fc.y_mode_cdf[size_group_lookup[bsize as usize] as usize];
+    let cdf = &self.fc.y_mode_cdf[size_group_lookup[bsize as usize] as usize];
     symbol_with_update!(self, w, mode as u32, cdf);
   }
 
@@ -735,10 +735,10 @@ impl<'a> ContextWriter<'a> {
     bs: BlockSize,
   ) {
     if bs.cfl_allowed() {
-      let cdf = &mut self.fc.uv_mode_cfl_cdf[y_mode as usize];
+      let cdf = &self.fc.uv_mode_cfl_cdf[y_mode as usize];
       symbol_with_update!(self, w, uv_mode as u32, cdf);
     } else {
-      let cdf = &mut self.fc.uv_mode_cdf[y_mode as usize];
+      let cdf = &self.fc.uv_mode_cdf[y_mode as usize];
       symbol_with_update!(self, w, uv_mode as u32, cdf);
     }
   }
@@ -751,7 +751,7 @@ impl<'a> ContextWriter<'a> {
       self,
       w,
       (angle + MAX_ANGLE_DELTA as i8) as u32,
-      &mut self.fc.angle_delta_cdf
+      &self.fc.angle_delta_cdf
         [mode as usize - PredictionMode::V_PRED as usize]
     );
   }
@@ -759,10 +759,13 @@ impl<'a> ContextWriter<'a> {
   pub fn write_use_filter_intra<W: Writer>(
     &mut self, w: &mut W, enable: bool, block_size: BlockSize,
   ) {
-    let cdf = &mut self.fc.filter_intra_cdfs[block_size as usize];
-    symbol_with_update!(self, w, enable as u32, cdf, 2);
+    let cdf = &self.fc.filter_intra_cdfs[block_size as usize];
+    symbol_with_update!(self, w, enable as u32, cdf);
   }
 
+  /// # Panics
+  ///
+  /// - If called with `enable: true` (not yet implemented
   pub fn write_use_palette_mode<W: Writer>(
     &mut self, w: &mut W, enable: bool, bsize: BlockSize, bo: TileBlockOffset,
     luma_mode: PredictionMode, chroma_mode: PredictionMode, xdec: usize,
@@ -776,15 +779,15 @@ impl<'a> ContextWriter<'a> {
 
     if luma_mode == PredictionMode::DC_PRED {
       let bsize_ctx = bsize.width_mi_log2() + bsize.height_mi_log2() - 2;
-      let cdf = &mut self.fc.palette_y_mode_cdfs[bsize_ctx][ctx_luma];
-      symbol_with_update!(self, w, enable as u32, cdf, 2);
+      let cdf = &self.fc.palette_y_mode_cdfs[bsize_ctx][ctx_luma];
+      symbol_with_update!(self, w, enable as u32, cdf);
     }
 
     if has_chroma(bo, bsize, xdec, ydec, cs)
       && chroma_mode == PredictionMode::DC_PRED
     {
-      let cdf = &mut self.fc.palette_uv_mode_cdfs[ctx_chroma];
-      symbol_with_update!(self, w, enable as u32, cdf, 2);
+      let cdf = &self.fc.palette_uv_mode_cdfs[ctx_chroma];
+      symbol_with_update!(self, w, enable as u32, cdf);
     }
   }
 
@@ -820,7 +823,7 @@ impl<'a> ContextWriter<'a> {
   fn find_matching_mv_and_update_weight(
     mv: MotionVector, mv_stack: &mut ArrayVec<CandidateMV, 9>, weight: u32,
   ) -> bool {
-    for mut mv_cand in mv_stack {
+    for mv_cand in mv_stack {
       if mv.row == mv_cand.this_mv.row && mv.col == mv_cand.this_mv.col {
         mv_cand.weight += weight;
         return true;
@@ -1006,7 +1009,7 @@ impl<'a> ContextWriter<'a> {
         let inc = cmp::min(-max_row_offs + row_offset + 1, cand.n4_h as isize);
         assert!(inc >= 0);
         weight = cmp::max(weight, inc as u32);
-        *processed_rows = (inc as isize) - row_offset - 1;
+        *processed_rows = inc - row_offset - 1;
       }
 
       if Self::add_ref_mv_candidate(
@@ -1072,7 +1075,7 @@ impl<'a> ContextWriter<'a> {
         let inc = cmp::min(-max_col_offs + col_offset + 1, cand.n4_w as isize);
         assert!(inc >= 0);
         weight = cmp::max(weight, inc as u32);
-        *processed_cols = (inc as isize) - col_offset - 1;
+        *processed_cols = inc - col_offset - 1;
       }
 
       if Self::add_ref_mv_candidate(
@@ -1114,11 +1117,12 @@ impl<'a> ContextWriter<'a> {
   }
 
   fn add_offset(mv_stack: &mut ArrayVec<CandidateMV, 9>) {
-    for mut cand_mv in mv_stack {
+    for cand_mv in mv_stack {
       cand_mv.weight += REF_CAT_LEVEL;
     }
   }
 
+  #[hawktracer(setup_mvref_list)]
   fn setup_mvref_list<T: Pixel>(
     &self, bo: TileBlockOffset, ref_frames: [RefType; 2],
     mv_stack: &mut ArrayVec<CandidateMV, 9>, bsize: BlockSize,
@@ -1210,8 +1214,7 @@ impl<'a> ContextWriter<'a> {
       row_match |= found_match;
     }
 
-    let nearest_match =
-      if row_match { 1 } else { 0 } + if col_match { 1 } else { 0 };
+    let nearest_match = usize::from(row_match) + usize::from(col_match);
 
     Self::add_offset(mv_stack);
 
@@ -1268,8 +1271,7 @@ impl<'a> ContextWriter<'a> {
       }
     }
 
-    let total_match =
-      if row_match { 1 } else { 0 } + if col_match { 1 } else { 0 };
+    let total_match = usize::from(row_match) + usize::from(col_match);
 
     assert!(total_match >= nearest_match);
 
@@ -1293,8 +1295,7 @@ impl<'a> ContextWriter<'a> {
       let h4 = bsize.height_mi().min(16).min(self.bc.blocks.rows() - bo.0.y);
       let num4x4 = w4.min(h4);
 
-      let passes =
-        if up_avail { 0 } else { 1 }..if left_avail { 2 } else { 1 };
+      let passes = i32::from(!up_avail)..=i32::from(left_avail);
 
       let mut ref_id_count: [usize; 2] = [0; 2];
       let mut ref_diff_count: [usize; 2] = [0; 2];
@@ -1403,18 +1404,21 @@ impl<'a> ContextWriter<'a> {
         * (8 * MI_SIZE) as isize
         + border_h;
       mv.this_mv.row =
-        (mv.this_mv.row as isize).max(mvy_min).min(mvy_max) as i16;
+        (mv.this_mv.row as isize).clamp(mvy_min, mvy_max) as i16;
       mv.this_mv.col =
-        (mv.this_mv.col as isize).max(mvx_min).min(mvx_max) as i16;
+        (mv.this_mv.col as isize).clamp(mvx_min, mvx_max) as i16;
       mv.comp_mv.row =
-        (mv.comp_mv.row as isize).max(mvy_min).min(mvy_max) as i16;
+        (mv.comp_mv.row as isize).clamp(mvy_min, mvy_max) as i16;
       mv.comp_mv.col =
-        (mv.comp_mv.col as isize).max(mvx_min).min(mvx_max) as i16;
+        (mv.comp_mv.col as isize).clamp(mvx_min, mvx_max) as i16;
     }
 
     mode_context
   }
 
+  /// # Panics
+  ///
+  /// - If the first ref frame is not set (`NONE_FRAME`)
   pub fn find_mvrefs<T: Pixel>(
     &self, bo: TileBlockOffset, ref_frames: [RefType; 2],
     mv_stack: &mut ArrayVec<CandidateMV, 9>, bsize: BlockSize,
@@ -1462,7 +1466,7 @@ impl<'a> ContextWriter<'a> {
   }
 
   #[inline]
-  pub fn ref_count_ctx(counts0: u8, counts1: u8) -> usize {
+  pub const fn ref_count_ctx(counts0: u8, counts1: u8) -> usize {
     if counts0 < counts1 {
       0
     } else if counts0 == counts1 {
@@ -1649,6 +1653,9 @@ impl<'a> ContextWriter<'a> {
     }
   }
 
+  /// # Panics
+  ///
+  /// - If `mode` is not an inter mode
   pub fn write_compound_mode<W: Writer>(
     &mut self, w: &mut W, mode: PredictionMode, ctx: usize,
   ) {
@@ -1681,7 +1688,7 @@ impl<'a> ContextWriter<'a> {
       PredictionMode::NEW_NEWMV => 7,
       _ => unreachable!(),
     };
-    symbol_with_update!(self, w, val, &mut self.fc.compound_mode_cdf[ctx]);
+    symbol_with_update!(self, w, val, &self.fc.compound_mode_cdf[ctx]);
   }
 
   pub fn write_inter_mode<W: Writer>(
@@ -1689,16 +1696,16 @@ impl<'a> ContextWriter<'a> {
   ) {
     use PredictionMode::{GLOBALMV, NEARESTMV, NEWMV};
     let newmv_ctx = ctx & NEWMV_CTX_MASK;
-    let cdf = &mut self.fc.newmv_cdf[newmv_ctx];
-    symbol_with_update!(self, w, (mode != NEWMV) as u32, cdf, 2);
+    let cdf = &self.fc.newmv_cdf[newmv_ctx];
+    symbol_with_update!(self, w, (mode != NEWMV) as u32, cdf);
     if mode != NEWMV {
       let zeromv_ctx = (ctx >> GLOBALMV_OFFSET) & GLOBALMV_CTX_MASK;
-      let cdf = &mut self.fc.zeromv_cdf[zeromv_ctx];
-      symbol_with_update!(self, w, (mode != GLOBALMV) as u32, cdf, 2);
+      let cdf = &self.fc.zeromv_cdf[zeromv_ctx];
+      symbol_with_update!(self, w, (mode != GLOBALMV) as u32, cdf);
       if mode != GLOBALMV {
         let refmv_ctx = (ctx >> REFMV_OFFSET) & REFMV_CTX_MASK;
-        let cdf = &mut self.fc.refmv_cdf[refmv_ctx];
-        symbol_with_update!(self, w, (mode != NEARESTMV) as u32, cdf, 2);
+        let cdf = &self.fc.refmv_cdf[refmv_ctx];
+        symbol_with_update!(self, w, (mode != NEARESTMV) as u32, cdf);
       }
     }
   }
@@ -1707,10 +1714,13 @@ impl<'a> ContextWriter<'a> {
   pub fn write_drl_mode<W: Writer>(
     &mut self, w: &mut W, drl_mode: bool, ctx: usize,
   ) {
-    let cdf = &mut self.fc.drl_cdfs[ctx];
-    symbol_with_update!(self, w, drl_mode as u32, cdf, 2);
+    let cdf = &self.fc.drl_cdfs[ctx];
+    symbol_with_update!(self, w, drl_mode as u32, cdf);
   }
 
+  /// # Panics
+  ///
+  /// - If the MV is invalid
   pub fn write_mv<W: Writer>(
     &mut self, w: &mut W, mv: MotionVector, ref_mv: MotionVector,
     mv_precision: MvSubpelPrecision,
@@ -1722,8 +1732,8 @@ impl<'a> ContextWriter<'a> {
       MotionVector { row: mv.row - ref_mv.row, col: mv.col - ref_mv.col };
     let j: MvJointType = av1_get_mv_joint(diff);
 
-    let cdf = &mut self.fc.nmv_context.joints_cdf;
-    symbol_with_update!(self, w, j as u32, cdf, 4);
+    let cdf = &self.fc.nmv_context.joints_cdf;
+    symbol_with_update!(self, w, j as u32, cdf);
 
     if mv_joint_vertical(j) {
       self.encode_mv_component(w, diff.row as i32, 0, mv_precision);
@@ -1739,17 +1749,16 @@ impl<'a> ContextWriter<'a> {
     let block = &self.bc.blocks[bo];
     let deltas_count = if multi { FRAME_LF_COUNT + planes - 3 } else { 1 };
     let deltas = &block.deblock_deltas[..deltas_count];
-    let cdf1 = &mut [self.fc.deblock_delta_cdf];
-    let cdfs = if multi {
-      &mut self.fc.deblock_delta_multi_cdf[..deltas_count]
-    } else {
-      cdf1
-    };
 
-    for (&delta, cdf) in deltas.iter().zip(cdfs.iter_mut()) {
-      let abs = delta.abs() as u32;
+    for (i, &delta) in deltas.iter().enumerate() {
+      let abs = delta.unsigned_abs() as u32;
+      let cdf = if multi {
+        &self.fc.deblock_delta_multi_cdf[i]
+      } else {
+        &self.fc.deblock_delta_cdf
+      };
 
-      symbol_with_update!(self, w, cmp::min(abs, DELTA_LF_SMALL), cdf, 4);
+      symbol_with_update!(self, w, cmp::min(abs, DELTA_LF_SMALL), cdf);
 
       if abs >= DELTA_LF_SMALL {
         let bits = msb(abs as i32 - 1) as u32;
@@ -1766,8 +1775,8 @@ impl<'a> ContextWriter<'a> {
     &mut self, w: &mut W, bo: TileBlockOffset, is_inter: bool,
   ) {
     let ctx = self.bc.intra_inter_context(bo);
-    let cdf = &mut self.fc.intra_inter_cdfs[ctx];
-    symbol_with_update!(self, w, is_inter as u32, cdf, 2);
+    let cdf = &self.fc.intra_inter_cdfs[ctx];
+    symbol_with_update!(self, w, is_inter as u32, cdf);
   }
 
   pub fn write_coeffs_lv_map<T: Coefficient, W: Writer>(
@@ -1793,7 +1802,7 @@ impl<'a> ContextWriter<'a> {
     let coeffs = &mut coeffs_storage.data;
     coeffs.extend(scan.iter().map(|&scan_idx| coeffs_in[scan_idx as usize]));
 
-    let mut cul_level = coeffs.iter().map(|c| u32::cast_from(c.abs())).sum();
+    let cul_level: u32 = coeffs.iter().map(|c| u32::cast_from(c.abs())).sum();
 
     let txs_ctx = Self::get_txsize_entropy_ctx(tx_size);
     let txb_ctx = self.bc.get_txb_ctx(
@@ -1808,8 +1817,8 @@ impl<'a> ContextWriter<'a> {
     );
 
     {
-      let cdf = &mut self.fc.txb_skip_cdf[txs_ctx][txb_ctx.txb_skip_ctx];
-      symbol_with_update!(self, w, (eob == 0) as u32, cdf, 2);
+      let cdf = &self.fc.txb_skip_cdf[txs_ctx][txb_ctx.txb_skip_ctx];
+      symbol_with_update!(self, w, (eob == 0) as u32, cdf);
     }
 
     if eob == 0 {
@@ -1824,7 +1833,7 @@ impl<'a> ContextWriter<'a> {
     self.txb_init_levels(coeffs_in, height, levels, height + TX_PAD_HOR);
 
     let tx_class = tx_type_to_class[tx_type as usize];
-    let plane_type = if plane == 0 { 0 } else { 1 } as usize;
+    let plane_type = usize::from(plane != 0);
 
     // Signal tx_type for luma plane only
     if plane == 0 {
@@ -1838,39 +1847,52 @@ impl<'a> ContextWriter<'a> {
       );
     }
 
-    // Encode EOB
+    self.encode_eob(eob, tx_size, tx_class, txs_ctx, plane_type, w);
+    self.encode_coeffs(
+      coeffs, levels, scan, eob, tx_size, tx_class, txs_ctx, plane_type, w,
+    );
+    let cul_level =
+      self.encode_coeff_signs(coeffs, w, plane_type, txb_ctx, cul_level);
+    self.bc.set_coeff_context(plane, bo, tx_size, xdec, ydec, cul_level as u8);
+    true
+  }
+
+  fn encode_eob<W: Writer>(
+    &mut self, eob: usize, tx_size: TxSize, tx_class: TxClass, txs_ctx: usize,
+    plane_type: usize, w: &mut W,
+  ) {
     let mut eob_extra: u32 = 0;
     let eob_pt = Self::get_eob_pos_token(eob, &mut eob_extra);
     let eob_multi_size: usize = tx_size.area_log2() - 4;
-    let eob_multi_ctx: usize = if tx_class == TX_CLASS_2D { 0 } else { 1 };
+    let eob_multi_ctx: usize = usize::from(tx_class != TX_CLASS_2D);
 
     match eob_multi_size {
       0 => {
-        let cdf = &mut self.fc.eob_flag_cdf16[plane_type][eob_multi_ctx];
+        let cdf = &self.fc.eob_flag_cdf16[plane_type][eob_multi_ctx];
         symbol_with_update!(self, w, eob_pt - 1, cdf);
       }
       1 => {
-        let cdf = &mut self.fc.eob_flag_cdf32[plane_type][eob_multi_ctx];
+        let cdf = &self.fc.eob_flag_cdf32[plane_type][eob_multi_ctx];
         symbol_with_update!(self, w, eob_pt - 1, cdf);
       }
       2 => {
-        let cdf = &mut self.fc.eob_flag_cdf64[plane_type][eob_multi_ctx];
+        let cdf = &self.fc.eob_flag_cdf64[plane_type][eob_multi_ctx];
         symbol_with_update!(self, w, eob_pt - 1, cdf);
       }
       3 => {
-        let cdf = &mut self.fc.eob_flag_cdf128[plane_type][eob_multi_ctx];
+        let cdf = &self.fc.eob_flag_cdf128[plane_type][eob_multi_ctx];
         symbol_with_update!(self, w, eob_pt - 1, cdf);
       }
       4 => {
-        let cdf = &mut self.fc.eob_flag_cdf256[plane_type][eob_multi_ctx];
+        let cdf = &self.fc.eob_flag_cdf256[plane_type][eob_multi_ctx];
         symbol_with_update!(self, w, eob_pt - 1, cdf);
       }
       5 => {
-        let cdf = &mut self.fc.eob_flag_cdf512[plane_type][eob_multi_ctx];
+        let cdf = &self.fc.eob_flag_cdf512[plane_type][eob_multi_ctx];
         symbol_with_update!(self, w, eob_pt - 1, cdf);
       }
       _ => {
-        let cdf = &mut self.fc.eob_flag_cdf1024[plane_type][eob_multi_ctx];
+        let cdf = &self.fc.eob_flag_cdf1024[plane_type][eob_multi_ctx];
         symbol_with_update!(self, w, eob_pt - 1, cdf);
       }
     }
@@ -1879,20 +1901,26 @@ impl<'a> ContextWriter<'a> {
 
     if eob_offset_bits > 0 {
       let mut eob_shift = eob_offset_bits - 1;
-      let mut bit: u32 =
-        if (eob_extra & (1 << eob_shift)) != 0 { 1 } else { 0 };
+      let mut bit: u32 = u32::from((eob_extra & (1 << eob_shift)) != 0);
       let cdf =
-        &mut self.fc.eob_extra_cdf[txs_ctx][plane_type][(eob_pt - 3) as usize];
-      symbol_with_update!(self, w, bit, cdf, 2);
+        &self.fc.eob_extra_cdf[txs_ctx][plane_type][(eob_pt - 3) as usize];
+      symbol_with_update!(self, w, bit, cdf);
       for i in 1..eob_offset_bits {
-        eob_shift = eob_offset_bits as u16 - 1 - i as u16;
-        bit = if (eob_extra & (1 << eob_shift)) != 0 { 1 } else { 0 };
+        eob_shift = eob_offset_bits - 1 - i;
+        bit = u32::from((eob_extra & (1 << eob_shift)) != 0);
         w.bit(bit as u16);
       }
     }
+  }
 
+  fn encode_coeffs<T: Coefficient, W: Writer>(
+    &mut self, coeffs: &[T], levels: &mut [u8], scan: &[u16], eob: usize,
+    tx_size: TxSize, tx_class: TxClass, txs_ctx: usize, plane_type: usize,
+    w: &mut W,
+  ) {
+    // SAFETY: We write to the array below before reading from it.
     let mut coeff_contexts: Aligned<[i8; MAX_CODED_TX_SQUARE]> =
-      Aligned::uninitialized();
+      unsafe { Aligned::uninitialized() };
 
     self.get_nz_map_contexts(
       levels,
@@ -1914,18 +1942,15 @@ impl<'a> ContextWriter<'a> {
         symbol_with_update!(
           self,
           w,
-          (cmp::min(u32::cast_from(level), 3) - 1) as u32,
-          &mut self.fc.coeff_base_eob_cdf[txs_ctx][plane_type]
-            [coeff_ctx as usize],
-          3
+          cmp::min(u32::cast_from(level), 3) - 1,
+          &self.fc.coeff_base_eob_cdf[txs_ctx][plane_type][coeff_ctx as usize]
         );
       } else {
         symbol_with_update!(
           self,
           w,
-          (cmp::min(u32::cast_from(level), 3)) as u32,
-          &mut self.fc.coeff_base_cdf[txs_ctx][plane_type][coeff_ctx as usize],
-          4
+          cmp::min(u32::cast_from(level), 3),
+          &self.fc.coeff_base_cdf[txs_ctx][plane_type][coeff_ctx as usize]
         );
       }
 
@@ -1939,9 +1964,9 @@ impl<'a> ContextWriter<'a> {
             break;
           }
           let k = cmp::min(base_range - idx, T::cast_from(BR_CDF_SIZE - 1));
-          let cdf = &mut self.fc.coeff_br_cdf
+          let cdf = &self.fc.coeff_br_cdf
             [txs_ctx.min(TxSize::TX_32X32 as usize)][plane_type][br_ctx];
-          symbol_with_update!(self, w, u32::cast_from(k), cdf, 4);
+          symbol_with_update!(self, w, u32::cast_from(k), cdf);
           if k < T::cast_from(BR_CDF_SIZE - 1) {
             break;
           }
@@ -1949,7 +1974,12 @@ impl<'a> ContextWriter<'a> {
         }
       }
     }
+  }
 
+  fn encode_coeff_signs<T: Coefficient, W: Writer>(
+    &mut self, coeffs: &[T], w: &mut W, plane_type: usize, txb_ctx: TXB_CTX,
+    orig_cul_level: u32,
+  ) -> u32 {
     // Loop to code all signs in the transform block,
     // starting with the sign of DC (if applicable)
     for (c, &v) in coeffs.iter().enumerate() {
@@ -1958,10 +1988,10 @@ impl<'a> ContextWriter<'a> {
       }
 
       let level = v.abs();
-      let sign = if v < T::cast_from(0) { 1 } else { 0 };
+      let sign = u32::from(v < T::cast_from(0));
       if c == 0 {
-        let cdf = &mut self.fc.dc_sign_cdf[plane_type][txb_ctx.dc_sign_ctx];
-        symbol_with_update!(self, w, sign, cdf, 2);
+        let cdf = &self.fc.dc_sign_cdf[plane_type][txb_ctx.dc_sign_ctx];
+        symbol_with_update!(self, w, sign, cdf);
       } else {
         w.bit(sign as u16);
       }
@@ -1973,11 +2003,11 @@ impl<'a> ContextWriter<'a> {
       }
     }
 
-    cul_level = cmp::min(COEFF_CONTEXT_MASK as u32, cul_level);
+    let mut new_cul_level =
+      cmp::min(COEFF_CONTEXT_MASK as u32, orig_cul_level);
 
-    BlockContext::set_dc_sign(&mut cul_level, i32::cast_from(coeffs[0]));
+    BlockContext::set_dc_sign(&mut new_cul_level, i32::cast_from(coeffs[0]));
 
-    self.bc.set_coeff_context(plane, bo, tx_size, xdec, ydec, cul_level as u8);
-    true
+    new_cul_level
   }
 }
