@@ -18,8 +18,7 @@ cfg_if::cfg_if! {
   }
 }
 
-use crate::context::CDFContextLog;
-use crate::util::{msb, ILog};
+use crate::context::{CDFContext, CDFContextLog, CDFOffset};
 use bitstream_io::{BigEndian, BitWrite, BitWriter};
 use std::io;
 
@@ -28,66 +27,67 @@ const EC_PROB_SHIFT: u32 = 6;
 const EC_MIN_PROB: u32 = 4;
 type ec_window = u32;
 
-/// Public trait interface to a bitstream Writer: a Counter can be
+/// Public trait interface to a bitstream `Writer`: a `Counter` can be
 /// used to count bits for cost analysis without actually storing
-/// anything (using a new::WriterCounter() as a Writer), to record
-/// tokens for later writing (using a new::WriterRecorder() as a
-/// Writer) to write actual final bits out using a range encoder
-/// (using a new::WriterEncoder() as a Writer).  A WriterRecorder's
-/// contents can be replayed into a WriterEncoder.
+/// anything (using a new `WriterCounter` as a `Writer`), to record
+/// tokens for later writing (using a new `WriterRecorder` as a
+/// `Writer`) to write actual final bits out using a range encoder
+/// (using a new `WriterEncoder` as a `Writer`).  A `WriterRecorder`'s
+/// contents can be replayed into a `WriterEncoder`.
 pub trait Writer {
-  /// Write a symbol s, using the passed in cdf reference; leaves cdf unchanged
-  fn symbol(&mut self, s: u32, cdf: &[u16]);
-  /// return approximate number of fractional bits in OD_BITRES
-  /// precision to write a symbol s using the passed in cdf reference;
-  /// leaves cdf unchanged
+  /// Write a symbol `s`, using the passed in cdf reference; leaves `cdf` unchanged
+  fn symbol<const CDF_LEN: usize>(&mut self, s: u32, cdf: &[u16; CDF_LEN]);
+  /// return approximate number of fractional bits in `OD_BITRES`
+  /// precision to write a symbol `s` using the passed in cdf reference;
+  /// leaves `cdf` unchanged
   fn symbol_bits(&self, s: u32, cdf: &[u16]) -> u32;
-  /// Write a symbol s, using the passed in cdf reference; updates the referenced cdf.
+  /// Write a symbol `s`, using the passed in cdf reference; updates the referenced cdf.
   fn symbol_with_update<const CDF_LEN: usize>(
-    &mut self, s: u32, cdf: &mut [u16; CDF_LEN], log: &mut CDFContextLog,
+    &mut self, s: u32, cdf: CDFOffset<CDF_LEN>, log: &mut CDFContextLog,
+    fc: &mut CDFContext,
   );
   /// Write a bool using passed in probability
   fn bool(&mut self, val: bool, f: u16);
-  /// Write a single bit with flat proability
+  /// Write a single bit with flat probability
   fn bit(&mut self, bit: u16);
-  /// Write literal bits with flat probability
+  /// Write literal `bits` with flat probability
   fn literal(&mut self, bits: u8, s: u32);
-  /// Write passed level as a golomb code
+  /// Write passed `level` as a golomb code
   fn write_golomb(&mut self, level: u32);
-  /// Write a value v in [0, n-1] quasi-uniformly
+  /// Write a value `v` in `[0, n-1]` quasi-uniformly
   fn write_quniform(&mut self, n: u32, v: u32);
-  /// Return fractional bits needed to write Write a value v in [0,
-  /// n-1] quasi-uniformly
+  /// Return fractional bits needed to write a value `v` in `[0, n-1]`
+  /// quasi-uniformly
   fn count_quniform(&self, n: u32, v: u32) -> u32;
-  /// Write symbol v in [0, n-1] with parameter k as finite subexponential
+  /// Write symbol `v` in `[0, n-1]` with parameter `k` as finite subexponential
   fn write_subexp(&mut self, n: u32, k: u8, v: u32);
-  /// Return fractional bits needed to write symbol v in [0, n-1] with
+  /// Return fractional bits needed to write symbol v in `[0, n-1]` with
   /// parameter k as finite subexponential
   fn count_subexp(&self, n: u32, k: u8, v: u32) -> u32;
-  /// Write symbol v in [0, n-1] with parameter k as finite
-  /// subexponential based on a reference ref also in [0, n-1].
+  /// Write symbol `v` in `[0, n-1]` with parameter `k` as finite
+  /// subexponential based on a reference `r` also in `[0, n-1]`.
   fn write_unsigned_subexp_with_ref(&mut self, v: u32, mx: u32, k: u8, r: u32);
-  /// Return fractional bits beed to write symbol v in [0, n-1] with
-  /// parameter k as finite subexponential based on a reference ref
-  /// also in [0, n-1].
+  /// Return fractional bits needed to write symbol `v` in `[0, n-1]` with
+  /// parameter `k` as finite subexponential based on a reference `r`
+  /// also in `[0, n-1]`.
   fn count_unsigned_subexp_with_ref(
     &self, v: u32, mx: u32, k: u8, r: u32,
   ) -> u32;
-  /// Write symbol v in [-(n-1), n-1] with parameter k as finite
-  /// subexponential based on a reference ref also in [-(n-1), n-1].
+  /// Write symbol v in `[-(n-1), n-1]` with parameter k as finite
+  /// subexponential based on a reference ref also in `[-(n-1), n-1]`.
   fn write_signed_subexp_with_ref(
     &mut self, v: i32, low: i32, high: i32, k: u8, r: i32,
   );
-  /// Return fractional bits needed to write symbol v in [-(n-1), n-1]
-  /// with parameter k as finite subexponential based on a reference
-  /// ref also in [-(n-1), n-1].
+  /// Return fractional bits needed to write symbol `v` in `[-(n-1), n-1]`
+  /// with parameter `k` as finite subexponential based on a reference
+  /// `r` also in `[-(n-1), n-1]`.
   fn count_signed_subexp_with_ref(
     &self, v: i32, low: i32, high: i32, k: u8, r: i32,
   ) -> u32;
   /// Return current length of range-coded bitstream in integer bits
   fn tell(&mut self) -> u32;
   /// Return current length of range-coded bitstream in fractional
-  /// bits with OD_BITRES decimal precision
+  /// bits with `OD_BITRES` decimal precision
   fn tell_frac(&mut self) -> u32;
   /// Save current point in coding/recording to a checkpoint
   fn checkpoint(&mut self) -> WriterCheckpoint;
@@ -97,15 +97,15 @@ pub trait Writer {
   fn add_bits_frac(&mut self, bits_frac: u32);
 }
 
-/// StorageBackend is an internal trait used to tie a specific Writer
-/// implementation's storage to the generic Writer.  It would be
+/// `StorageBackend` is an internal trait used to tie a specific `Writer`
+/// implementation's storage to the generic `Writer`.  It would be
 /// private, but Rust is deprecating 'private trait in a public
 /// interface' support.
 pub trait StorageBackend {
   /// Store partially-computed range code into given storage backend
   fn store(&mut self, fl: u16, fh: u16, nms: u16);
-  /// Return byte-length of encoded stream to date
-  fn stream_bytes(&mut self) -> usize;
+  /// Return bit-length of encoded stream to date
+  fn stream_bits(&mut self) -> usize;
   /// Backend implementation of checkpoint to pass through Writer interface
   fn checkpoint(&mut self) -> WriterCheckpoint;
   /// Backend implementation of rollback to pass through Writer interface
@@ -130,16 +130,16 @@ pub struct WriterBase<S> {
 
 #[derive(Debug, Clone)]
 pub struct WriterCounter {
-  /// Bytes that would be shifted out to date
-  bytes: usize,
+  /// Bits that would be shifted out to date
+  bits: usize,
 }
 
 #[derive(Debug, Clone)]
 pub struct WriterRecorder {
   /// Storage for tokens
   storage: Vec<(u16, u16, u16)>,
-  /// Bytes that would be shifted out to date
-  bytes: usize,
+  /// Bits that would be shifted out to date
+  bits: usize,
 }
 
 #[derive(Debug, Clone)]
@@ -152,8 +152,10 @@ pub struct WriterEncoder {
 
 #[derive(Clone)]
 pub struct WriterCheckpoint {
-  /// Byte length coded/recorded to date
-  stream_bytes: usize,
+  /// Stream length coded/recorded to date, in the unit used by the Writer,
+  /// which may be bytes or bits. This depends on the assumption
+  /// that a Writer will only ever restore its own Checkpoint.
+  stream_size: usize,
   /// To be defined by backend
   backend_var: usize,
   /// Saved number of values in the current range.
@@ -165,23 +167,23 @@ pub struct WriterCheckpoint {
 /// Constructor for a counting Writer
 impl WriterCounter {
   #[inline]
-  pub fn new() -> WriterBase<WriterCounter> {
-    WriterBase::new(WriterCounter { bytes: 0 })
+  pub const fn new() -> WriterBase<WriterCounter> {
+    WriterBase::new(WriterCounter { bits: 0 })
   }
 }
 
 /// Constructor for a recording Writer
 impl WriterRecorder {
   #[inline]
-  pub fn new() -> WriterBase<WriterRecorder> {
-    WriterBase::new(WriterRecorder { storage: Vec::new(), bytes: 0 })
+  pub const fn new() -> WriterBase<WriterRecorder> {
+    WriterBase::new(WriterRecorder { storage: Vec::new(), bits: 0 })
   }
 }
 
 /// Constructor for a encoding Writer
 impl WriterEncoder {
   #[inline]
-  pub fn new() -> WriterBase<WriterEncoder> {
+  pub const fn new() -> WriterBase<WriterEncoder> {
     WriterBase::new(WriterEncoder { precarry: Vec::new(), low: 0 })
   }
 }
@@ -192,33 +194,30 @@ impl StorageBackend for WriterBase<WriterCounter> {
   #[inline]
   fn store(&mut self, fl: u16, fh: u16, nms: u16) {
     let (_l, r) = self.lr_compute(fl, fh, nms);
-    let d = 16 - r.ilog();
-    let mut s = self.cnt + (d as i16);
-
-    self.s.bytes += (s >= 0) as usize + (s >= 8) as usize;
-    s -= 8 * ((s >= 0) as i16 + (s >= 8) as i16);
+    let d = r.leading_zeros() as usize;
 
+    self.s.bits += d;
     self.rng = r << d;
-    self.cnt = s;
   }
   #[inline]
-  fn stream_bytes(&mut self) -> usize {
-    self.s.bytes
+  fn stream_bits(&mut self) -> usize {
+    self.s.bits
   }
   #[inline]
   fn checkpoint(&mut self) -> WriterCheckpoint {
     WriterCheckpoint {
-      stream_bytes: self.s.bytes,
+      stream_size: self.s.bits,
       backend_var: 0,
       rng: self.rng,
+      // We do not use `cnt` within Counter, but setting it here allows the compiler
+      // to do a 32-bit merged load/store.
       cnt: self.cnt,
     }
   }
   #[inline]
   fn rollback(&mut self, checkpoint: &WriterCheckpoint) {
     self.rng = checkpoint.rng;
-    self.cnt = checkpoint.cnt;
-    self.s.bytes = checkpoint.stream_bytes;
+    self.s.bits = checkpoint.stream_size;
   }
 }
 
@@ -230,24 +229,20 @@ impl StorageBackend for WriterBase<WriterRecorder> {
   #[inline]
   fn store(&mut self, fl: u16, fh: u16, nms: u16) {
     let (_l, r) = self.lr_compute(fl, fh, nms);
-    let d = 16 - r.ilog();
-    let mut s = self.cnt + (d as i16);
-
-    self.s.bytes += (s >= 0) as usize + (s >= 8) as usize;
-    s -= 8 * ((s >= 0) as i16 + (s >= 8) as i16);
+    let d = r.leading_zeros() as usize;
 
+    self.s.bits += d;
     self.rng = r << d;
-    self.cnt = s;
     self.s.storage.push((fl, fh, nms));
   }
   #[inline]
-  fn stream_bytes(&mut self) -> usize {
-    self.s.bytes
+  fn stream_bits(&mut self) -> usize {
+    self.s.bits
   }
   #[inline]
   fn checkpoint(&mut self) -> WriterCheckpoint {
     WriterCheckpoint {
-      stream_bytes: self.s.bytes,
+      stream_size: self.s.bits,
       backend_var: self.s.storage.len(),
       rng: self.rng,
       cnt: self.cnt,
@@ -257,7 +252,7 @@ impl StorageBackend for WriterBase<WriterRecorder> {
   fn rollback(&mut self, checkpoint: &WriterCheckpoint) {
     self.rng = checkpoint.rng;
     self.cnt = checkpoint.cnt;
-    self.s.bytes = checkpoint.stream_bytes;
+    self.s.bits = checkpoint.stream_size;
     self.s.storage.truncate(checkpoint.backend_var);
   }
 }
@@ -271,7 +266,7 @@ impl StorageBackend for WriterBase<WriterEncoder> {
     let (l, r) = self.lr_compute(fl, fh, nms);
     let mut low = l + self.s.low;
     let mut c = self.cnt;
-    let d = 16 - r.ilog();
+    let d = r.leading_zeros() as usize;
     let mut s = c + (d as i16);
 
     if s >= 0 {
@@ -292,13 +287,13 @@ impl StorageBackend for WriterBase<WriterEncoder> {
     self.cnt = s;
   }
   #[inline]
-  fn stream_bytes(&mut self) -> usize {
-    self.s.precarry.len()
+  fn stream_bits(&mut self) -> usize {
+    self.s.precarry.len() * 8
   }
   #[inline]
   fn checkpoint(&mut self) -> WriterCheckpoint {
     WriterCheckpoint {
-      stream_bytes: self.s.precarry.len(),
+      stream_size: self.s.precarry.len(),
       backend_var: self.s.low as usize,
       rng: self.rng,
       cnt: self.cnt,
@@ -308,7 +303,7 @@ impl StorageBackend for WriterBase<WriterEncoder> {
     self.rng = checkpoint.rng;
     self.cnt = checkpoint.cnt;
     self.s.low = checkpoint.backend_var as ec_window;
-    self.s.precarry.truncate(checkpoint.stream_bytes);
+    self.s.precarry.truncate(checkpoint.stream_size);
   }
 }
 
@@ -318,40 +313,36 @@ impl<S> WriterBase<S> {
   /// Internal constructor called by the subtypes that implement the
   /// actual encoder and Recorder.
   #[inline]
+  #[cfg(not(feature = "desync_finder"))]
+  const fn new(storage: S) -> Self {
+    WriterBase { rng: 0x8000, cnt: -9, fake_bits_frac: 0, s: storage }
+  }
+
+  #[inline]
+  #[cfg(feature = "desync_finder")]
   fn new(storage: S) -> Self {
-    #[cfg(feature = "desync_finder")]
-    {
-      WriterBase {
-        rng: 0x8000,
-        cnt: -9,
-        debug: std::env::var_os("RAV1E_DEBUG").is_some(),
-        fake_bits_frac: 0,
-        s: storage,
-      }
-    }
-    #[cfg(not(feature = "desync_finder"))]
-    {
-      WriterBase { rng: 0x8000, cnt: -9, fake_bits_frac: 0, s: storage }
+    WriterBase {
+      rng: 0x8000,
+      cnt: -9,
+      debug: std::env::var_os("RAV1E_DEBUG").is_some(),
+      fake_bits_frac: 0,
+      s: storage,
     }
   }
 
   /// Compute low and range values from token cdf values and local state
-  fn lr_compute(&mut self, fl: u16, fh: u16, nms: u16) -> (ec_window, u16) {
-    let u: u32;
-    let v: u32;
-    let mut r = self.rng as u32;
+  const fn lr_compute(&self, fl: u16, fh: u16, nms: u16) -> (ec_window, u16) {
+    let r = self.rng as u32;
     debug_assert!(32768 <= r);
-    if fl < 32768 {
-      u = (((r >> 8) * (fl as u32 >> EC_PROB_SHIFT)) >> (7 - EC_PROB_SHIFT))
-        + EC_MIN_PROB * nms as u32;
-      v = (((r >> 8) * (fh as u32 >> EC_PROB_SHIFT)) >> (7 - EC_PROB_SHIFT))
-        + EC_MIN_PROB * (nms - 1) as u32;
-      (r - u, (u - v) as u16)
-    } else {
-      r -= (((r >> 8) * (fh as u32 >> EC_PROB_SHIFT)) >> (7 - EC_PROB_SHIFT))
-        + EC_MIN_PROB * (nms - 1) as u32;
-      (0, r as u16)
+    let mut u = (((r >> 8) * (fl as u32 >> EC_PROB_SHIFT))
+      >> (7 - EC_PROB_SHIFT))
+      + EC_MIN_PROB * nms as u32;
+    if fl >= 32768 {
+      u = r;
     }
+    let v = (((r >> 8) * (fh as u32 >> EC_PROB_SHIFT)) >> (7 - EC_PROB_SHIFT))
+      + EC_MIN_PROB * (nms - 1) as u32;
+    (r - u, (u - v) as u16)
   }
 
   /// Given the current total integer number of bits used and the current value of
@@ -387,7 +378,7 @@ impl<S> WriterBase<S> {
     nbits - l
   }
 
-  fn recenter(r: u32, v: u32) -> u32 {
+  const fn recenter(r: u32, v: u32) -> u32 {
     if v > (r << 1) {
       v
     } else if v >= r {
@@ -431,7 +422,7 @@ impl WriterBase<WriterRecorder> {
     self.rng = 0x8000;
     self.cnt = -9;
     self.s.storage.truncate(0);
-    self.s.bytes = 0;
+    self.s.bits = 0;
   }
 }
 
@@ -482,7 +473,8 @@ impl WriterBase<WriterEncoder> {
   }
 }
 
-/// Generic/shared implementation for Writers with StorageBackends (ie, Encoders and Recorders)
+/// Generic/shared implementation for `Writer`s with `StorageBackend`s
+/// (ie, `Encoder`s and `Recorder`s)
 impl<S> Writer for WriterBase<S>
 where
   WriterBase<S>: StorageBackend,
@@ -493,11 +485,12 @@ where
   fn bool(&mut self, val: bool, f: u16) {
     debug_assert!(0 < f);
     debug_assert!(f < 32768);
-    self.symbol(if val { 1 } else { 0 }, &[f, 0]);
+    self.symbol(u32::from(val), &[f, 0]);
   }
   /// Encode a single boolean value.
-  /// `val`: The value to encode (false or true).
-  /// `f`: The probability that the val is true, scaled by 32768.
+  ///
+  /// - `val`: The value to encode (`false` or `true`).
+  /// - `f`: The probability that the `val` is `true`, scaled by `32768`.
   fn bit(&mut self, bit: u16) {
     self.bool(bit == 1, 16384);
   }
@@ -507,44 +500,54 @@ where
   }
   /// Encode a literal bitstring, bit by bit in MSB order, with flat
   /// probability.
-  /// 'bits': Length of bitstring
-  /// 's': Bit string to encode
+  ///
+  /// - 'bits': Length of bitstring
+  /// - 's': Bit string to encode
   fn literal(&mut self, bits: u8, s: u32) {
     for bit in (0..bits).rev() {
       self.bit((1 & (s >> bit)) as u16);
     }
   }
   /// Encodes a symbol given a cumulative distribution function (CDF) table in Q15.
-  /// `s`: The index of the symbol to encode.
-  /// `cdf`: The CDF, such that symbol s falls in the range
+  ///
+  /// - `s`: The index of the symbol to encode.
+  /// - `cdf`: The CDF, such that symbol s falls in the range
   ///        `[s > 0 ? cdf[s - 1] : 0, cdf[s])`.
   ///       The values must be monotonically non-decreasing, and the last value
   ///       must be greater than 32704. There should be at most 16 values.
   ///       The lower 6 bits of the last value hold the count.
   #[inline(always)]
-  fn symbol(&mut self, s: u32, cdf: &[u16]) {
+  fn symbol<const CDF_LEN: usize>(&mut self, s: u32, cdf: &[u16; CDF_LEN]) {
     debug_assert!(cdf[cdf.len() - 1] < (1 << EC_PROB_SHIFT));
     let s = s as usize;
     debug_assert!(s < cdf.len());
     // The above is stricter than the following overflow check: s <= cdf.len()
     let nms = cdf.len() - s;
-    let fl = if s > 0 { unsafe { *cdf.get_unchecked(s - 1) } } else { 32768 };
+    let fl = if s > 0 {
+      // SAFETY: We asserted that s is less than the length of the cdf
+      unsafe { *cdf.get_unchecked(s - 1) }
+    } else {
+      32768
+    };
+    // SAFETY: We asserted that s is less than the length of the cdf
     let fh = unsafe { *cdf.get_unchecked(s) };
     debug_assert!((fh >> EC_PROB_SHIFT) <= (fl >> EC_PROB_SHIFT));
     debug_assert!(fl <= 32768);
     self.store(fl, fh, nms as u16);
   }
   /// Encodes a symbol given a cumulative distribution function (CDF)
-  /// table in Q15, then updates the CDF probabilities to relect we've
+  /// table in Q15, then updates the CDF probabilities to reflect we've
   /// written one more symbol 's'.
-  /// `s`: The index of the symbol to encode.
-  /// `cdf`: The CDF, such that symbol s falls in the range
+  ///
+  /// - `s`: The index of the symbol to encode.
+  /// - `cdf`: The CDF, such that symbol s falls in the range
   ///        `[s > 0 ? cdf[s - 1] : 0, cdf[s])`.
   ///       The values must be monotonically non-decreasing, and the last value
   ///       must be greater 32704. There should be at most 16 values.
   ///       The lower 6 bits of the last value hold the count.
   fn symbol_with_update<const CDF_LEN: usize>(
-    &mut self, s: u32, cdf: &mut [u16; CDF_LEN], log: &mut CDFContextLog,
+    &mut self, s: u32, cdf: CDFOffset<CDF_LEN>, log: &mut CDFContextLog,
+    fc: &mut CDFContext,
   ) {
     #[cfg(feature = "desync_finder")]
     {
@@ -552,15 +555,16 @@ where
         self.print_backtrace(s);
       }
     }
-    log.push(cdf);
+    let cdf = log.push(fc, cdf);
     self.symbol(s, cdf);
 
     update_cdf(cdf, s);
   }
   /// Returns approximate cost for a symbol given a cumulative
   /// distribution function (CDF) table and current write state.
-  /// `s`: The index of the symbol to encode.
-  /// `cdf`: The CDF, such that symbol s falls in the range
+  ///
+  /// - `s`: The index of the symbol to encode.
+  /// - `cdf`: The CDF, such that symbol s falls in the range
   ///        `[s > 0 ? cdf[s - 1] : 0, cdf[s])`.
   ///       The values must be monotonically non-decreasing, and the last value
   ///       must be greater than 32704. There should be at most 16 values.
@@ -571,7 +575,7 @@ where
     debug_assert!(32768 <= self.rng);
     let rng = (self.rng >> 8) as u32;
     let fh = cdf[s as usize] as u32 >> EC_PROB_SHIFT;
-    let r = if s > 0 {
+    let r: u32 = if s > 0 {
       let fl = cdf[s as usize - 1] as u32 >> EC_PROB_SHIFT;
       ((rng * fl) >> (7 - EC_PROB_SHIFT)) - ((rng * fh) >> (7 - EC_PROB_SHIFT))
         + EC_MIN_PROB
@@ -584,7 +588,7 @@ where
 
     // The 9 here counteracts the offset of -9 baked into cnt.  Don't include a termination bit.
     let pre = Self::frac_compute((self.cnt + 9) as u32, self.rng as u32);
-    let d = 16 - r.ilog();
+    let d = r.leading_zeros() - 16;
     let mut c = self.cnt;
     let mut sh = c + (d as i16);
     if sh >= 0 {
@@ -600,7 +604,8 @@ where
     Self::frac_compute((bits + sh + 9) as u32, r << d) - pre
   }
   /// Encode a golomb to the bitstream.
-  /// 'level': passed in value to encode
+  ///
+  /// - 'level': passed in value to encode
   fn write_golomb(&mut self, level: u32) {
     let x = level + 1;
     let length = 32 - x.leading_zeros();
@@ -613,12 +618,12 @@ where
       self.bit(((x >> i) & 0x01) as u16);
     }
   }
-  /// Write a value v in [0, n-1] quasi-uniformly
-  /// n: size of interval
-  /// v: value to encode
+  /// Write a value `v` in `[0, n-1]` quasi-uniformly
+  /// - `n`: size of interval
+  /// - `v`: value to encode
   fn write_quniform(&mut self, n: u32, v: u32) {
     if n > 1 {
-      let l = msb(n as i32) as u8 + 1;
+      let l = 32 - n.leading_zeros() as u8;
       let m = (1 << l) - n;
       if v < m {
         self.literal(l - 1, v);
@@ -628,13 +633,13 @@ where
       }
     }
   }
-  /// Returns QOD_BITRES bits for a value v in [0, n-1] quasi-uniformly
-  /// n: size of interval
-  /// v: value to encode
+  /// Returns `QOD_BITRES` bits for a value `v` in `[0, n-1]` quasi-uniformly
+  /// - `n`: size of interval
+  /// - `v`: value to encode
   fn count_quniform(&self, n: u32, v: u32) -> u32 {
     let mut bits = 0;
     if n > 1 {
-      let l = (msb(n as i32) + 1) as u32;
+      let l = 32 - n.leading_zeros();
       let m = (1 << l) - n;
       bits += (l - 1) << OD_BITRES;
       if v >= m {
@@ -643,10 +648,11 @@ where
     }
     bits
   }
-  /// Write symbol v in [0, n-1] with parameter k as finite subexponential
-  /// n: size of interval
-  /// k: 'parameter'
-  /// v: value to encode
+  /// Write symbol `v` in `[0, n-1]` with parameter `k` as finite subexponential
+  ///
+  /// - `n`: size of interval
+  /// - `k`: "parameter"
+  /// - `v`: value to encode
   fn write_subexp(&mut self, n: u32, k: u8, v: u32) {
     let mut i = 0;
     let mut mk = 0;
@@ -669,10 +675,12 @@ where
       }
     }
   }
-  /// Returns QOD_BITRES bits for symbol v in [0, n-1] with parameter k as finite subexponential
-  /// n: size of interval
-  /// k: 'parameter'
-  /// v: value to encode
+  /// Returns `QOD_BITRES` bits for symbol `v` in `[0, n-1]` with parameter `k`
+  /// as finite subexponential
+  ///
+  /// - `n`: size of interval
+  /// - `k`: "parameter"
+  /// - `v`: value to encode
   fn count_subexp(&self, n: u32, k: u8, v: u32) -> u32 {
     let mut i = 0;
     let mut mk = 0;
@@ -697,12 +705,13 @@ where
     }
     bits
   }
-  /// Write symbol v in [0, n-1] with parameter k as finite
-  /// subexponential based on a reference ref also in [0, n-1].
-  /// v: value to encode
-  /// n: size of interval
-  /// k: 'parameter'
-  /// r: reference
+  /// Write symbol `v` in `[0, n-1]` with parameter `k` as finite
+  /// subexponential based on a reference `r` also in `[0, n-1]`.
+  ///
+  /// - `v`: value to encode
+  /// - `n`: size of interval
+  /// - `k`: "parameter"
+  /// - `r`: reference
   fn write_unsigned_subexp_with_ref(&mut self, v: u32, n: u32, k: u8, r: u32) {
     if (r << 1) <= n {
       self.write_subexp(n, k, Self::recenter(r, v));
@@ -710,12 +719,14 @@ where
       self.write_subexp(n, k, Self::recenter(n - 1 - r, n - 1 - v));
     }
   }
-  /// Returns QOD_BITRES bits for symbol v in [0, n-1] with parameter k as finite
-  /// subexponential based on a reference ref also in [0, n-1].
-  /// v: value to encode
-  /// n: size of interval
-  /// k: 'parameter'
-  /// r: reference
+  /// Returns `QOD_BITRES` bits for symbol `v` in `[0, n-1]`
+  /// with parameter `k` as finite subexponential based on a
+  /// reference `r` also in `[0, n-1]`.
+  ///
+  /// - `v`: value to encode
+  /// - `n`: size of interval
+  /// - `k`: "parameter"
+  /// - `r`: reference
   fn count_unsigned_subexp_with_ref(
     &self, v: u32, n: u32, k: u8, r: u32,
   ) -> u32 {
@@ -725,12 +736,13 @@ where
       self.count_subexp(n, k, Self::recenter(n - 1 - r, n - 1 - v))
     }
   }
-  /// Write symbol v in [-(n-1), n-1] with parameter k as finite
-  /// subexponential based on a reference ref also in [-(n-1), n-1].
-  /// v: value to encode
-  /// n: size of interval
-  /// k: 'parameter'
-  /// r: reference
+  /// Write symbol `v` in `[-(n-1), n-1]` with parameter `k` as finite
+  /// subexponential based on a reference `r` also in `[-(n-1), n-1]`.
+  ///
+  /// - `v`: value to encode
+  /// - `n`: size of interval
+  /// - `k`: "parameter"
+  /// - `r`: reference
   fn write_signed_subexp_with_ref(
     &mut self, v: i32, low: i32, high: i32, k: u8, r: i32,
   ) {
@@ -741,12 +753,15 @@ where
       (r - low) as u32,
     );
   }
-  /// Returns QOD_BITRES bits for symbol v in [-(n-1), n-1] with parameter k as finite
-  /// subexponential based on a reference ref also in [-(n-1), n-1].
-  /// v: value to encode
-  /// n: size of interval
-  /// k: 'parameter'
-  /// r: reference
+  /// Returns `QOD_BITRES` bits for symbol `v` in `[-(n-1), n-1]`
+  /// with parameter `k` as finite subexponential based on a
+  /// reference `r` also in `[-(n-1), n-1]`.
+  ///
+  /// - `v`: value to encode
+  /// - `n`: size of interval
+  /// - `k`: "parameter"
+  /// - `r`: reference
+
   fn count_signed_subexp_with_ref(
     &self, v: i32, low: i32, high: i32, k: u8, r: i32,
   ) -> u32 {
@@ -760,20 +775,22 @@ where
   /// Returns the number of bits "used" by the encoded symbols so far.
   /// This same number can be computed in either the encoder or the
   /// decoder, and is suitable for making coding decisions.  The value
-  /// will be the same whether using an Encoder or Recorder.
+  /// will be the same whether using an `Encoder` or `Recorder`.
+  ///
   /// Return: The integer number of bits.
   ///         This will always be slightly larger than the exact value (e.g., all
   ///          rounding error is in the positive direction).
   fn tell(&mut self) -> u32 {
     // The 10 here counteracts the offset of -9 baked into cnt, and adds 1 extra
     // bit, which we reserve for terminating the stream.
-    (((self.stream_bytes() * 8) as i32) + (self.cnt as i32) + 10) as u32
+    (((self.stream_bits()) as i32) + (self.cnt as i32) + 10) as u32
       + (self.fake_bits_frac >> 8)
   }
   /// Returns the number of bits "used" by the encoded symbols so far.
   /// This same number can be computed in either the encoder or the
   /// decoder, and is suitable for making coding decisions. The value
-  /// will be the same whether using an Encoder or Recorder.
+  /// will be the same whether using an `Encoder` or `Recorder`.
+  ///
   /// Return: The number of bits scaled by `2**OD_BITRES`.
   ///         This will always be slightly larger than the exact value (e.g., all
   ///          rounding error is in the positive direction).
@@ -781,14 +798,15 @@ where
     Self::frac_compute(self.tell(), self.rng as u32) + self.fake_bits_frac
   }
   /// Save current point in coding/recording to a checkpoint that can
-  /// be restored later.  A WriterCheckpoint can be generated for an
-  /// Encoder or Recorder, but can only be used to rollback the Writer
+  /// be restored later.  A `WriterCheckpoint` can be generated for an
+  /// `Encoder` or `Recorder`, but can only be used to rollback the `Writer`
   /// instance from which it was generated.
   fn checkpoint(&mut self) -> WriterCheckpoint {
     StorageBackend::checkpoint(self)
   }
-  /// Roll back a given Writer to the state saved in the WriterCheckpoint
-  /// 'wc': Saved Writer state/posiiton to restore
+  /// Roll back a given `Writer` to the state saved in the `WriterCheckpoint`
+  ///
+  /// - 'wc': Saved `Writer` state/posiiton to restore
   fn rollback(&mut self, wc: &WriterCheckpoint) {
     StorageBackend::rollback(self, wc)
   }
@@ -797,13 +815,25 @@ where
 pub trait BCodeWriter {
   fn recenter_nonneg(&mut self, r: u16, v: u16) -> u16;
   fn recenter_finite_nonneg(&mut self, n: u16, r: u16, v: u16) -> u16;
+  /// # Errors
+  ///
+  /// - Returns `std::io::Error` if the writer cannot be written to.
   fn write_quniform(&mut self, n: u16, v: u16) -> Result<(), std::io::Error>;
+  /// # Errors
+  ///
+  /// - Returns `std::io::Error` if the writer cannot be written to.
   fn write_subexpfin(
     &mut self, n: u16, k: u16, v: u16,
   ) -> Result<(), std::io::Error>;
+  /// # Errors
+  ///
+  /// - Returns `std::io::Error` if the writer cannot be written to.
   fn write_refsubexpfin(
     &mut self, n: u16, k: u16, r: i16, v: i16,
   ) -> Result<(), std::io::Error>;
+  /// # Errors
+  ///
+  /// - Returns `std::io::Error` if the writer cannot be written to.
   fn write_s_refsubexpfin(
     &mut self, n: u16, k: u16, r: i16, v: i16,
   ) -> Result<(), std::io::Error>;
@@ -831,7 +861,7 @@ impl<W: io::Write> BCodeWriter for BitWriter<W, BigEndian> {
   }
   fn write_quniform(&mut self, n: u16, v: u16) -> Result<(), std::io::Error> {
     if n > 1 {
-      let l = msb(n as i32) as u8 + 1;
+      let l = 16 - n.leading_zeros() as u8;
       let m = (1 << l) - n;
       if v < m {
         self.write(l as u32 - 1, v)
@@ -903,7 +933,7 @@ pub(crate) fn cdf_to_pdf<const CDF_LEN: usize>(
 pub(crate) mod rust {
   // Function to update the CDF for Writer calls that do so.
   #[inline]
-  pub fn update_cdf(cdf: &mut [u16], val: u32) {
+  pub fn update_cdf<const N: usize>(cdf: &mut [u16; N], val: u32) {
     use crate::context::CDF_LEN_MAX;
     let nsymbs = cdf.len();
     let mut rate = 3 + (nsymbs >> 1).min(2);
@@ -973,7 +1003,6 @@ mod test {
       assert!(rng <= 65536);
       let d = rng.leading_zeros() - 16;
       //let d = 16 - (32-rng.leading_zeros());
-      //msb(rng) = 31-rng.leading_zeros();
       self.cnt -= d as i16;
       /*This is equivalent to shifting in 1's instead of 0's.*/
       self.dif = ((dif + 1) << d) - 1;
@@ -1042,12 +1071,12 @@ mod test {
 
     let mut r = Reader::new(&b);
 
-    assert_eq!(r.bool(1), false);
-    assert_eq!(r.bool(2), true);
-    assert_eq!(r.bool(3), false);
-    assert_eq!(r.bool(1), true);
-    assert_eq!(r.bool(2), true);
-    assert_eq!(r.bool(3), false);
+    assert!(!r.bool(1));
+    assert!(r.bool(2));
+    assert!(!r.bool(3));
+    assert!(r.bool(1));
+    assert!(r.bool(2));
+    assert!(!r.bool(3));
   }
 
   #[test]
@@ -1107,15 +1136,15 @@ mod test {
     let mut r = Reader::new(&b);
 
     assert_eq!(r.symbol(&cdf), 0);
-    assert_eq!(r.bool(2), true);
+    assert!(r.bool(2));
     assert_eq!(r.symbol(&cdf), 0);
-    assert_eq!(r.bool(2), true);
+    assert!(r.bool(2));
     assert_eq!(r.symbol(&cdf), 0);
-    assert_eq!(r.bool(2), true);
+    assert!(r.bool(2));
     assert_eq!(r.symbol(&cdf), 1);
-    assert_eq!(r.bool(1), true);
+    assert!(r.bool(1));
     assert_eq!(r.symbol(&cdf), 1);
-    assert_eq!(r.bool(2), false);
+    assert!(!r.bool(2));
     assert_eq!(r.symbol(&cdf), 1);
     assert_eq!(r.symbol(&cdf), 2);
     assert_eq!(r.symbol(&cdf), 2);
