@@ -11,7 +11,10 @@ use crate::api::color::ChromaSampling;
 use crate::api::ContextInner;
 use crate::encoder::TEMPORAL_DELIMITER;
 use crate::quantize::{ac_q, dc_q, select_ac_qi, select_dc_qi};
-use crate::util::{clamp, ILog, Pixel};
+use crate::util::{
+  bexp64, bexp_q24, blog64, clamp, q24_to_q57, q57, q57_to_q24, Pixel,
+};
+use rust_hawktracer::*;
 use std::cmp;
 
 // The number of frame sub-types for which we track distinct parameters.
@@ -106,218 +109,6 @@ const Q_MODEL_MUL: [i64; 4] = [
   0,
 ];
 
-// Convert an integer into a Q57 fixed-point fraction.
-// The integer must be in the range -64 to 63, inclusive.
-pub(crate) const fn q57(v: i32) -> i64 {
-  // TODO: Add assert if it ever becomes possible to do in a const function.
-  (v as i64) << 57
-}
-
-#[rustfmt::skip]
-const ATANH_LOG2: &[i64; 32] = &[
-  0x32B8_0347_3F7A_D0F4, 0x2F2A_71BD_4E25_E916, 0x2E68_B244_BB93_BA06,
-  0x2E39_FB91_98CE_62E4, 0x2E2E_683F_6856_5C8F, 0x2E2B_850B_E207_7FC1,
-  0x2E2A_CC58_FE7B_78DB, 0x2E2A_9E2D_E52F_D5F2, 0x2E2A_92A3_38D5_3EEC,
-  0x2E2A_8FC0_8F5E_19B6, 0x2E2A_8F07_E51A_485E, 0x2E2A_8ED9_BA8A_F388,
-  0x2E2A_8ECE_2FE7_384A, 0x2E2A_8ECB_4D3E_4B1A, 0x2E2A_8ECA_9494_0FE8,
-  0x2E2A_8ECA_6669_811D, 0x2E2A_8ECA_5ADE_DD6A, 0x2E2A_8ECA_57FC_347E,
-  0x2E2A_8ECA_5743_8A43, 0x2E2A_8ECA_5715_5FB4, 0x2E2A_8ECA_5709_D510,
-  0x2E2A_8ECA_5706_F267, 0x2E2A_8ECA_5706_39BD, 0x2E2A_8ECA_5706_0B92,
-  0x2E2A_8ECA_5706_0008, 0x2E2A_8ECA_5705_FD25, 0x2E2A_8ECA_5705_FC6C,
-  0x2E2A_8ECA_5705_FC3E, 0x2E2A_8ECA_5705_FC33, 0x2E2A_8ECA_5705_FC30,
-  0x2E2A_8ECA_5705_FC2F, 0x2E2A_8ECA_5705_FC2F
-];
-
-// Computes the binary exponential of logq57.
-// input: a log base 2 in Q57 format.
-// output: a 64 bit integer in Q0 (no fraction).
-// TODO: Mark const once we can use local variables in a const function.
-pub(crate) fn bexp64(logq57: i64) -> i64 {
-  let ipart = (logq57 >> 57) as i32;
-  if ipart < 0 {
-    return 0;
-  }
-  if ipart >= 63 {
-    return 0x7FFF_FFFF_FFFF_FFFF;
-  }
-  // z is the fractional part of the log in Q62 format.
-  // We need 1 bit of headroom since the magnitude can get larger than 1
-  //  during the iteration, and a sign bit.
-  let mut z = logq57 - q57(ipart);
-  let mut w: i64;
-  if z != 0 {
-    // Rust has 128 bit multiplies, so it should be possible to do this
-    //  faster without losing accuracy.
-    z <<= 5;
-    // w is the exponential in Q61 format (since it also needs headroom and can
-    //  get as large as 2.0); we could get another bit if we dropped the sign,
-    //  but we'll recover that bit later anyway.
-    // Ideally this should start out as
-    //   \lim_{n->\infty} 2^{61}/\product_{i=1}^n \sqrt{1-2^{-2i}}
-    //  but in order to guarantee convergence we have to repeat iterations 4,
-    //  13 (=3*4+1), and 40 (=3*13+1, etc.), so it winds up somewhat larger.
-    w = 0x26A3_D0E4_01DD_846D;
-    let mut i: i64 = 0;
-    loop {
-      let mask = -((z < 0) as i64);
-      w += ((w >> (i + 1)) + mask) ^ mask;
-      z -= (ATANH_LOG2[i as usize] + mask) ^ mask;
-      // Repeat iteration 4.
-      if i >= 3 {
-        break;
-      }
-      z *= 2;
-      i += 1;
-    }
-    loop {
-      let mask = -((z < 0) as i64);
-      w += ((w >> (i + 1)) + mask) ^ mask;
-      z -= (ATANH_LOG2[i as usize] + mask) ^ mask;
-      // Repeat iteration 13.
-      if i >= 12 {
-        break;
-      }
-      z *= 2;
-      i += 1;
-    }
-    while i < 32 {
-      let mask = -((z < 0) as i64);
-      w += ((w >> (i + 1)) + mask) ^ mask;
-      z = (z - ((ATANH_LOG2[i as usize] + mask) ^ mask)) * 2;
-      i += 1;
-    }
-    // Skip the remaining iterations unless we really require that much
-    //  precision.
-    // We could have bailed out earlier for smaller iparts, but that would
-    //  require initializing w from a table, as the limit doesn't converge to
-    //  61-bit precision until n=30.
-    let mut wlo: i32 = 0;
-    if ipart > 30 {
-      // For these iterations, we just update the low bits, as the high bits
-      //  can't possibly be affected.
-      // OD_ATANH_LOG2 has also converged (it actually did so one iteration
-      //  earlier, but that's no reason for an extra special case).
-      loop {
-        let mask = -((z < 0) as i64);
-        wlo += (((w >> i) + mask) ^ mask) as i32;
-        z -= (ATANH_LOG2[31] + mask) ^ mask;
-        // Repeat iteration 40.
-        if i >= 39 {
-          break;
-        }
-        z *= 2;
-        i += 1;
-      }
-      while i < 61 {
-        let mask = -((z < 0) as i64);
-        wlo += (((w >> i) + mask) ^ mask) as i32;
-        z = (z - ((ATANH_LOG2[31] + mask) ^ mask)) * 2;
-        i += 1;
-      }
-    }
-    w = (w << 1) + (wlo as i64);
-  } else {
-    w = 1i64 << 62;
-  }
-  if ipart < 62 {
-    w = ((w >> (61 - ipart)) + 1) >> 1;
-  }
-  w
-}
-
-// Computes the binary log of w.
-// input: a 64-bit integer in Q0 (no fraction).
-// output: a 64-bit log in Q57.
-// TODO: Mark const once we can use local variables in a const function.
-fn blog64(w: i64) -> i64 {
-  let mut w = w;
-  if w <= 0 {
-    return -1;
-  }
-  let ipart = w.ilog() as i32 - 1;
-  if ipart > 61 {
-    w >>= ipart - 61;
-  } else {
-    w <<= 61 - ipart;
-  }
-  // z is the fractional part of the log in Q61 format.
-  let mut z: i64 = 0;
-  if (w & (w - 1)) != 0 {
-    // Rust has 128 bit multiplies, so it should be possible to do this
-    //  faster without losing accuracy.
-    // x and y are the cosh() and sinh(), respectively, in Q61 format.
-    // We are computing z = 2*atanh(y/x) = 2*atanh((w - 1)/(w + 1)).
-    let mut x = w + (1i64 << 61);
-    let mut y = w - (1i64 << 61);
-    for i in 0..4 {
-      let mask = -((y < 0) as i64);
-      z += ((ATANH_LOG2[i as usize] >> i) + mask) ^ mask;
-      let u = x >> (i + 1);
-      x -= ((y >> (i + 1)) + mask) ^ mask;
-      y -= (u + mask) ^ mask;
-    }
-    // Repeat iteration 4.
-    for i in 3..13 {
-      let mask = -((y < 0) as i64);
-      z += ((ATANH_LOG2[i as usize] >> i) + mask) ^ mask;
-      let u = x >> (i + 1);
-      x -= ((y >> (i + 1)) + mask) ^ mask;
-      y -= (u + mask) ^ mask;
-    }
-    // Repeat iteration 13.
-    for i in 12..32 {
-      let mask = -((y < 0) as i64);
-      z += ((ATANH_LOG2[i as usize] >> i) + mask) ^ mask;
-      let u = x >> (i + 1);
-      x -= ((y >> (i + 1)) + mask) ^ mask;
-      y -= (u + mask) ^ mask;
-    }
-    // OD_ATANH_LOG2 has converged.
-    for i in 32..40 {
-      let mask = -((y < 0) as i64);
-      z += ((ATANH_LOG2[31] >> i) + mask) ^ mask;
-      let u = x >> (i + 1);
-      x -= ((y >> (i + 1)) + mask) ^ mask;
-      y -= (u + mask) ^ mask;
-    }
-    // Repeat iteration 40.
-    for i in 39..62 {
-      let mask = -((y < 0) as i64);
-      z += ((ATANH_LOG2[31] >> i) + mask) ^ mask;
-      let u = x >> (i + 1);
-      x -= ((y >> (i + 1)) + mask) ^ mask;
-      y -= (u + mask) ^ mask;
-    }
-    z = (z + 8) >> 4;
-  }
-  q57(ipart) + z
-}
-
-// Converts a Q57 fixed-point fraction to Q24 by rounding.
-const fn q57_to_q24(v: i64) -> i32 {
-  (((v >> 32) + 1) >> 1) as i32
-}
-
-// Converts a Q24 fixed-point fraction to Q57.
-const fn q24_to_q57(v: i32) -> i64 {
-  (v as i64) << 33
-}
-
-// Binary exponentiation of a log_scale with 24-bit fractional precision and
-//  saturation.
-// log_scale: A binary logarithm in Q24 format.
-// Return: The binary exponential in Q24 format, saturated to 2**47 - 1 if
-//  log_scale was too large.
-fn bexp_q24(log_scale: i32) -> i64 {
-  if log_scale < 23 << 24 {
-    let ret = bexp64(((log_scale as i64) << 33) + q57(24));
-    if ret < (1i64 << 47) - 1 {
-      return ret;
-    }
-  }
-  (1i64 << 47) - 1
-}
-
 #[rustfmt::skip]
 const ROUGH_TAN_LOOKUP: &[u16; 18] = &[
      0,   358,   722,  1098,  1491,  1910,
@@ -337,7 +128,6 @@ pub struct IIRBessel2 {
 
 // alpha is Q24 in the range [0,0.5).
 // The return value is 5.12.
-// TODO: Mark const once we can use local variables in a const function.
 fn warp_alpha(alpha: i32) -> i32 {
   let i = ((alpha * 36) >> 24).min(16);
   let t0 = ROUGH_TAN_LOOKUP[i as usize];
@@ -736,8 +526,9 @@ impl QuantizerParameters {
   fn new_from_log_q(
     log_base_q: i64, log_target_q: i64, bit_depth: usize,
     chroma_sampling: ChromaSampling, is_intra: bool,
+    log_isqrt_mean_scale: i64,
   ) -> QuantizerParameters {
-    let scale = q57(QSCALE + bit_depth as i32 - 8);
+    let scale = log_isqrt_mean_scale + q57(QSCALE + bit_depth as i32 - 8);
 
     let mut log_q_y = log_target_q;
     if !is_intra && bit_depth == 8 {
@@ -747,14 +538,17 @@ impl QuantizerParameters {
     }
 
     let quantizer = bexp64(log_q_y + scale);
-    let (offset_u, offset_v) = chroma_offset(log_q_y, chroma_sampling);
+    let (offset_u, offset_v) =
+      chroma_offset(log_q_y + log_isqrt_mean_scale, chroma_sampling);
     let mono = chroma_sampling == ChromaSampling::Cs400;
     let log_q_u = log_q_y + offset_u;
     let log_q_v = log_q_y + offset_v;
     let quantizer_u = bexp64(log_q_u + scale);
     let quantizer_v = bexp64(log_q_v + scale);
     let lambda = (::std::f64::consts::LN_2 / 6.0)
-      * ((log_target_q as f64) * Q57_SQUARE_EXP_SCALE).exp();
+      * (((log_target_q + log_isqrt_mean_scale) as f64)
+        * Q57_SQUARE_EXP_SCALE)
+        .exp();
 
     let scale = |q| bexp64((log_target_q - q) * 2 + q57(16)) as f64 / 65536.;
     let dist_scale = [scale(log_q_y), scale(log_q_u), scale(log_q_v)];
@@ -764,7 +558,7 @@ impl QuantizerParameters {
     // delta_q only gets 6 bits + a sign bit, so it can differ by 63 at most.
     let min_qi = base_q_idx.saturating_sub(63).max(1);
     let max_qi = base_q_idx.saturating_add(63).min(255);
-    let clamp_qi = |qi: u8| qi.max(min_qi).min(max_qi);
+    let clamp_qi = |qi: u8| qi.clamp(min_qi, max_qi);
 
     QuantizerParameters {
       log_base_q,
@@ -786,20 +580,6 @@ impl QuantizerParameters {
   }
 }
 
-// The parameters that are required by twopass_out().
-// We need a reference to the enclosing ContextInner to compute these, but
-//  twopass_out() cannot take such a reference, since it needs a &mut self
-//  reference to do its job, and RCState is contained inside ContextInner.
-// In practice we don't modify anything in RCState until after we're finished
-//  reading from ContextInner, but Rust's borrow checker does not have a way to
-//  express that.
-// There's probably a cleaner way to do this, but going with something simple
-//  for now, since this is not exposed in the public API.
-pub(crate) struct TwoPassOutParams {
-  pub pass1_log_base_q: i64,
-  done_processing: bool,
-}
-
 impl RCState {
   pub fn new(
     frame_width: i32, frame_height: i32, framerate_num: i64,
@@ -935,13 +715,15 @@ impl RCState {
       bit_depth,
       chroma_sampling,
       fti == 0,
+      0,
     )
   }
 
   // TODO: Separate quantizers for Cb and Cr.
+  #[hawktracer(select_qi)]
   pub(crate) fn select_qi<T: Pixel>(
     &self, ctx: &ContextInner<T>, output_frameno: u64, fti: usize,
-    maybe_prev_log_base_q: Option<i64>,
+    maybe_prev_log_base_q: Option<i64>, log_isqrt_mean_scale: i64,
   ) -> QuantizerParameters {
     // Is rate control active?
     if self.target_bitrate <= 0 {
@@ -957,6 +739,7 @@ impl RCState {
         bit_depth,
         chroma_sampling,
         fti == 0,
+        log_isqrt_mean_scale,
       )
     } else {
       let mut nframes: [i32; FRAME_NSUBTYPES + 1] = [0; FRAME_NSUBTYPES + 1];
@@ -1106,7 +889,7 @@ impl RCState {
         }
       }
       // If we've been missing our target, add a penalty term.
-      let rate_bias = (self.rate_bias / (self.nencoded_frames as i64 + 100))
+      let rate_bias = (self.rate_bias / (self.nencoded_frames + 100))
         * (reservoir_frames as i64);
       // rate_total is the total bits available over the next
       //  reservoir_tus TUs.
@@ -1128,14 +911,14 @@ impl RCState {
       let bit_depth = ctx.config.bit_depth;
       let chroma_sampling = ctx.config.chroma_sampling;
       // TODO: Proper handling of lossless.
-      let mut log_qlo = blog64(ac_q(self.ac_qi_min, 0, bit_depth) as i64)
+      let mut log_qlo = blog64(ac_q(self.ac_qi_min, 0, bit_depth).get() as i64)
         - q57(QSCALE + bit_depth as i32 - 8);
       // The AC quantizer tables map to values larger than the DC quantizer
       //  tables, so we use that as the upper bound to make sure we can use
       //  the full table if needed.
-      let mut log_qhi =
-        blog64(ac_q(self.maybe_ac_qi_max.unwrap_or(255), 0, bit_depth) as i64)
-          - q57(QSCALE + bit_depth as i32 - 8);
+      let mut log_qhi = blog64(
+        ac_q(self.maybe_ac_qi_max.unwrap_or(255), 0, bit_depth).get() as i64,
+      ) - q57(QSCALE + bit_depth as i32 - 8);
       let mut log_base_q = (log_qlo + log_qhi) >> 1;
       while log_qlo < log_qhi {
         // Count bits contributed by each frame type using the model.
@@ -1255,6 +1038,7 @@ impl RCState {
         bit_depth,
         chroma_sampling,
         fti == 0,
+        log_isqrt_mean_scale,
       )
     }
   }
@@ -1270,10 +1054,10 @@ impl RCState {
 
     // We use the AC quantizer as the source quantizer since its quantizer
     //  tables have unique entries, while the DC tables do not.
-    let ac_quantizer = ac_q(base_qi as u8, 0, bit_depth) as i64;
+    let ac_quantizer = ac_q(base_qi, 0, bit_depth).get() as i64;
     // Pick the nearest DC entry since an exact match may be unavailable.
     let dc_qi = select_dc_qi(ac_quantizer, bit_depth);
-    let dc_quantizer = dc_q(dc_qi as u8, 0, bit_depth) as i64;
+    let dc_quantizer = dc_q(dc_qi, 0, bit_depth).get() as i64;
     // Get the log quantizers as Q57.
     let log_ac_q = blog64(ac_quantizer) - q57(QSCALE + bit_depth as i32 - 8);
     let log_dc_q = blog64(dc_quantizer) - q57(QSCALE + bit_depth as i32 - 8);
@@ -1285,6 +1069,7 @@ impl RCState {
     (log_base_q, log_q)
   }
 
+  #[hawktracer(RCState_update_state)]
   pub fn update_state(
     &mut self, bits: i64, fti: usize, show_frame: bool, log_target_q: i64,
     trial: bool, droppable: bool,
@@ -1447,11 +1232,11 @@ impl RCState {
     dropped
   }
 
-  pub fn needs_trial_encode(&self, fti: usize) -> bool {
+  pub const fn needs_trial_encode(&self, fti: usize) -> bool {
     self.target_bitrate > 0 && self.nframes[fti] == 0
   }
 
-  pub(crate) fn ready(&self) -> bool {
+  pub(crate) const fn ready(&self) -> bool {
     match self.twopass_state {
       PASS_SINGLE => true,
       PASS_1 => self.pass1_data_retrieved,
@@ -1473,26 +1258,19 @@ impl RCState {
     cur_pos
   }
 
-  pub(crate) fn get_twopass_out_params<T: Pixel>(
+  pub(crate) fn select_pass1_log_base_q<T: Pixel>(
     &self, ctx: &ContextInner<T>, output_frameno: u64,
-  ) -> TwoPassOutParams {
-    let mut pass1_log_base_q = 0;
-    let mut done_processing = false;
-    if !self.pass1_data_retrieved {
-      if self.twopass_state == PASS_SINGLE {
-        pass1_log_base_q = self
-          .select_qi(ctx, output_frameno, FRAME_SUBTYPE_I, None)
-          .log_base_q;
-      }
-    } else {
-      done_processing = ctx.done_processing();
-    }
-    TwoPassOutParams { pass1_log_base_q, done_processing }
+  ) -> i64 {
+    assert_eq!(self.twopass_state, PASS_SINGLE);
+    self.select_qi(ctx, output_frameno, FRAME_SUBTYPE_I, None, 0).log_base_q
   }
 
   // Initialize the first pass and emit a placeholder summary
-  pub(crate) fn init_first_pass(&mut self, pass1_log_base_q: i64) {
-    if self.twopass_state == PASS_SINGLE {
+  pub(crate) fn init_first_pass(
+    &mut self, maybe_pass1_log_base_q: Option<i64>,
+  ) {
+    if let Some(pass1_log_base_q) = maybe_pass1_log_base_q {
+      assert_eq!(self.twopass_state, PASS_SINGLE);
       // Pick first-pass qi for scale calculations.
       self.pass1_log_base_q = pass1_log_base_q;
     } else {
@@ -1561,19 +1339,17 @@ impl RCState {
     &self.pass1_buffer[..cur_pos]
   }
 
-  // Initialize the first pass, emit either summary or frame-specific data
-  // depending on the previous call
+  // Emit either summary or frame-specific data depending on the previous call
   pub(crate) fn twopass_out(
-    &mut self, params: TwoPassOutParams,
+    &mut self, done_processing: bool,
   ) -> Option<&[u8]> {
     if !self.pass1_data_retrieved {
       if self.twopass_state != PASS_1 && self.twopass_state != PASS_2_PLUS_1 {
-        self.init_first_pass(params.pass1_log_base_q);
         Some(self.emit_placeholder_summary())
       } else {
         self.emit_frame_data()
       }
-    } else if params.done_processing && !self.pass1_summary_retrieved {
+    } else if done_processing && !self.pass1_summary_retrieved {
       Some(self.emit_summary())
     } else {
       // The data for this frame has already been retrieved.
@@ -1674,7 +1450,7 @@ impl RCState {
       return 0;
     }
     if self.frame_metrics.is_empty() {
-      return if self.pass2_data_ready { 0 } else { 1 };
+      return i32::from(!self.pass2_data_ready);
     }
     let mut cur_scale_window_nframes = 0;
     let mut cur_nframes_left = 0;
@@ -1684,8 +1460,7 @@ impl RCState {
     }
 
     (self.reservoir_frame_delay - self.scale_window_ntus)
-      .max(0)
-      .min(cur_nframes_left - cur_scale_window_nframes)
+      .clamp(0, cur_nframes_left - cur_scale_window_nframes)
   }
 
   pub(crate) fn parse_frame_data_packet(
@@ -1809,8 +1584,7 @@ impl RCState {
               }
               frames_needed = (self.reservoir_frame_delay
                 - self.scale_window_ntus)
-                .max(0)
-                .min(cur_nframes_left - cur_scale_window_nframes);
+                .clamp(0, cur_nframes_left - cur_scale_window_nframes);
               // Clear the buffer for the next frame.
               self.des.pass2_buffer_fill = 0;
             } else {
@@ -1871,32 +1645,3 @@ impl RCState {
     Ok(consumed)
   }
 }
-
-#[cfg(test)]
-mod test {
-  use super::{bexp64, blog64};
-
-  #[test]
-  fn blog64_vectors() {
-    assert!(blog64(1793) == 0x159dc71e24d32daf);
-    assert!(blog64(0x678dde6e5fd29f05) == 0x7d6373ad151ca685);
-  }
-
-  #[test]
-  fn bexp64_vectors() {
-    assert!(bexp64(0x159dc71e24d32daf) == 1793);
-    assert!((bexp64(0x7d6373ad151ca685) - 0x678dde6e5fd29f05).abs() < 29);
-  }
-
-  #[test]
-  fn blog64_bexp64_round_trip() {
-    for a in 1..=std::u16::MAX as i64 {
-      let b = std::i64::MAX / a;
-      let (log_a, log_b, log_ab) = (blog64(a), blog64(b), blog64(a * b));
-      assert!((log_a + log_b - log_ab).abs() < 4);
-      assert!(bexp64(log_a) == a);
-      assert!((bexp64(log_b) - b).abs() < 128);
-      assert!((bexp64(log_ab) - a * b).abs() < 128);
-    }
-  }
-}
