@@ -11,10 +11,12 @@ use super::*;
 
 use crate::context::*;
 use crate::encoder::*;
+use crate::me::WriteGuardMEStats;
 use crate::util::*;
 
 use std::iter::FusedIterator;
 use std::marker::PhantomData;
+use std::ops::DerefMut;
 
 pub const MAX_TILE_WIDTH: usize = 4096;
 pub const MAX_TILE_AREA: usize = 4096 * 2304;
@@ -27,8 +29,8 @@ pub const MAX_TILE_RATE: f64 = 4096f64 * 2176f64 * 60f64 * 1.1;
 /// This stores everything necessary to split a frame into tiles, and write
 /// headers fields into the bitstream.
 ///
-/// The method tile_iter_mut() actually provides tiled views of FrameState
-/// and FrameBlocks.
+/// The method `tile_iter_mut()` actually provides tiled views of `FrameState`
+/// and `FrameBlocks`.
 #[derive(Debug, Clone, Copy)]
 pub struct TilingInfo {
   pub frame_width: usize,
@@ -48,6 +50,9 @@ pub struct TilingInfo {
 }
 
 impl TilingInfo {
+  /// # Panics
+  ///
+  /// Panics if the resulting tile sizes would be too large.
   pub fn from_target_tiles(
     sb_size_log2: usize, frame_width: usize, frame_height: usize,
     frame_rate: f64, tile_cols_log2: usize, tile_rows_log2: usize,
@@ -88,7 +93,7 @@ impl TilingInfo {
     );
 
     let tile_cols_log2 =
-      tile_cols_log2.max(min_tile_cols_log2).min(max_tile_cols_log2);
+      tile_cols_log2.clamp(min_tile_cols_log2, max_tile_cols_log2);
     let tile_width_sb_pre =
       sb_cols.align_power_of_two_and_shift(tile_cols_log2);
 
@@ -129,8 +134,7 @@ impl TilingInfo {
       };
     let tile_rows_log2 = tile_rows_log2
       .max(min_tile_rows_log2)
-      .max(min_tile_rows_ratelimit_log2)
-      .min(max_tile_rows_log2);
+      .clamp(min_tile_rows_ratelimit_log2, max_tile_rows_log2);
     let tile_height_sb = sb_rows.align_power_of_two_and_shift(tile_rows_log2);
 
     let rows = (frame_height_sb + tile_height_sb - 1) / tile_height_sb;
@@ -173,30 +177,33 @@ impl TilingInfo {
   /// Split frame-level structures into tiles
   ///
   /// Provide mutable tiled views of frame-level structures.
-  pub fn tile_iter_mut<'a, 'b, T: Pixel>(
-    &self, fs: &'a mut FrameState<T>, fb: &'b mut FrameBlocks,
-  ) -> TileContextIterMut<'a, 'b, T> {
-    TileContextIterMut { ti: *self, fs, fb, next: 0, phantom: PhantomData }
+  pub fn tile_iter_mut<'a, T: Pixel>(
+    &self, fs: &'a mut FrameState<T>, fb: &'a mut FrameBlocks,
+  ) -> TileContextIterMut<'a, T> {
+    let afs = fs as *mut _;
+    let afb = fb as *mut _;
+    let frame_me_stats = fs.frame_me_stats.write().expect("poisoned lock");
+    TileContextIterMut { ti: *self, fs: afs, fb: afb, next: 0, frame_me_stats }
   }
 }
 
 /// Container for all tiled views
-pub struct TileContextMut<'a, 'b, T: Pixel> {
+pub struct TileContextMut<'a, T: Pixel> {
   pub ts: TileStateMut<'a, T>,
-  pub tb: TileBlocksMut<'b>,
+  pub tb: TileBlocksMut<'a>,
 }
 
 /// Iterator over tiled views
-pub struct TileContextIterMut<'a, 'b, T: Pixel> {
+pub struct TileContextIterMut<'a, T: Pixel> {
   ti: TilingInfo,
   fs: *mut FrameState<T>,
   fb: *mut FrameBlocks,
+  frame_me_stats: WriteGuardMEStats<'a>,
   next: usize,
-  phantom: PhantomData<(&'a mut FrameState<T>, &'b mut FrameBlocks)>,
 }
 
-impl<'a, 'b, T: Pixel> Iterator for TileContextIterMut<'a, 'b, T> {
-  type Item = TileContextMut<'a, 'b, T>;
+impl<'a, T: Pixel> Iterator for TileContextIterMut<'a, T> {
+  type Item = TileContextMut<'a, T>;
 
   fn next(&mut self) -> Option<Self::Item> {
     if self.next < self.ti.rows * self.ti.cols {
@@ -204,7 +211,16 @@ impl<'a, 'b, T: Pixel> Iterator for TileContextIterMut<'a, 'b, T> {
       let tile_row = self.next / self.ti.cols;
       let ctx = TileContextMut {
         ts: {
+          // SAFETY: Multiple tiles mutably access this struct.
+          // The dimensions must be configured correctly to ensure
+          // the tiles do not overlap.
           let fs = unsafe { &mut *self.fs };
+          // SAFETY: ditto
+          let frame_me_stats = unsafe {
+            let len = self.frame_me_stats.len();
+            let ptr = self.frame_me_stats.as_mut_ptr();
+            std::slice::from_raw_parts_mut(ptr, len)
+          };
           let sbo = PlaneSuperBlockOffset(SuperBlockOffset {
             x: tile_col * self.ti.tile_width_sb,
             y: tile_row * self.ti.tile_height_sb,
@@ -215,9 +231,19 @@ impl<'a, 'b, T: Pixel> Iterator for TileContextIterMut<'a, 'b, T> {
           let tile_height = self.ti.tile_height_sb << self.ti.sb_size_log2;
           let width = tile_width.min(self.ti.frame_width - x);
           let height = tile_height.min(self.ti.frame_height - y);
-          TileStateMut::new(fs, sbo, self.ti.sb_size_log2, width, height)
+          TileStateMut::new(
+            fs,
+            sbo,
+            self.ti.sb_size_log2,
+            width,
+            height,
+            frame_me_stats,
+          )
         },
         tb: {
+          // SAFETY: Multiple tiles mutably access this struct.
+          // The dimensions must be configured correctly to ensure
+          // the tiles do not overlap.
           let fb = unsafe { &mut *self.fb };
           let tile_width_mi =
             self.ti.tile_width_sb << (self.ti.sb_size_log2 - MI_SIZE_LOG2);
@@ -243,8 +269,8 @@ impl<'a, 'b, T: Pixel> Iterator for TileContextIterMut<'a, 'b, T> {
   }
 }
 
-impl<T: Pixel> ExactSizeIterator for TileContextIterMut<'_, '_, T> {}
-impl<T: Pixel> FusedIterator for TileContextIterMut<'_, '_, T> {}
+impl<T: Pixel> ExactSizeIterator for TileContextIterMut<'_, T> {}
+impl<T: Pixel> FusedIterator for TileContextIterMut<'_, T> {}
 
 #[cfg(test)]
 pub mod test {
@@ -504,7 +530,7 @@ pub mod test {
   }
 
   #[inline]
-  fn b_area(region: &TileBlocksMut<'_>) -> (usize, usize, usize, usize) {
+  const fn b_area(region: &TileBlocksMut<'_>) -> (usize, usize, usize, usize) {
     (region.x(), region.y(), region.cols(), region.rows())
   }
 
@@ -744,10 +770,10 @@ pub mod test {
 
     // check that writes on tiled views affected the underlying motion vectors
 
-    let me_stats = &fs.frame_me_stats[0];
+    let me_stats = &fs.frame_me_stats.read().unwrap()[0];
     assert_eq!(MotionVector { col: 42, row: 38 }, me_stats[5][8].mv);
 
-    let me_stats = &fs.frame_me_stats[2];
+    let me_stats = &fs.frame_me_stats.read().unwrap()[2];
     let mix = (128 >> MI_SIZE_LOG2) + 3;
     let miy = (64 >> MI_SIZE_LOG2) + 2;
     assert_eq!(MotionVector { col: 2, row: 14 }, me_stats[miy][mix].mv);
